/* 
   devolo_usb.c: dLAN USB driver for linux. 

   Only for use with dLAN adaptors from devolo.
   Copyright 2007 devolo AG

*/
    
/*
 *   Include
 */
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/usb.h>
#include <linux/module.h>
#include <linux/ethtool.h>
#include <asm/uaccess.h>
#include <linux/config.h>
#include <linux/kmod.h>
#include <linux/random.h>
#include <asm/unaligned.h>
#include <linux/netdevice.h>
#include <linux/rtnetlink.h>
#include "devolo_usb.h"


static int multicast_filter_limit = 32;

/*
 *   List of supported vendor_id/device_id
 */
static struct usb_device_id plUsb_ids[] = {
  {
    USB_DEVICE (USB_INTELLON_VENDOR_ID, USB_INTELLON_PRODUCT_ID),
  },
  {},	       
};

#if defined(MODULE_LICENSE)
MODULE_LICENSE ("devolo AG");
#endif


MODULE_AUTHOR (DRIVER_AUTHOR);
MODULE_DESCRIPTION (DRIVER_DESC);
MODULE_DEVICE_TABLE (usb, plUsb_ids);

static void
ctrl_callback (struct urb * urb)
{
  devoloUSB_t *devolo = urb->context;

  if (!devolo)
    return;

  switch (urb->status)
    {
    case 0:
      break;
    case -EINPROGRESS:
      return;
    case -ENOENT:
      break;
    default:
      warn("%s: status %d",__FUNCTION__, urb->status);
    }

  wake_up (&devolo->ctrl_wait);

}


/*
 *   Function that send packet to upper level
 */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static void devolo_read_bulk_callback (struct urb *urb, struct pt_regs * unused)
#else
static void devolo_read_bulk_callback (struct urb *urb)
#endif
{
  devoloUSB_t *devolo = urb->context;
  struct net_device *net;

  int count = urb->actual_length;
  int res;
  struct sk_buff *skb;  
  
  if (!devolo || !(devolo->flags & DEVOLO_RUNNING))
    return;
  
  net = devolo->net;
  if (!netif_device_present (net))
    return;

  if (devolo->flags & DEVOLO_RX_BUSY){
    /*
     *  Update stats
     */
    devolo->stats.rx_errors++;
    return;
  }
  devolo->flags |= DEVOLO_RX_BUSY;
  
  switch (urb->status){
  case 0:
    break;
  case -ETIMEDOUT:
    devolo->flags &= ~DEVOLO_RX_BUSY;
    break;
  case -EPIPE:
    /* stall or disconnect */
    warn("%s: no rx stall recovery", net->name);
    return;
  case -ENOENT:
  case -ECONNRESET:
  case -ESHUTDOWN:
    return;
  default:
    goto goon;
  }
  
  if (!count)
    goto goon;

  if (!(skb = dev_alloc_skb (count)))
    goto goon;

  skb->dev = net;

  eth_copy_and_sum (skb, devolo->rx_buff, count, 0);
  /* bug 1180*/
  skb_put (skb, count-2);

  skb->protocol = eth_type_trans (skb, net);

  /*
   *  Sending packet to upper level
   */
  netif_rx (skb);

  /*
   *  Update stats
   */
  devolo->stats.rx_packets++;
  devolo->stats.rx_bytes += count;

 goon:
  FILL_BULK_URB (devolo->rx_urb, devolo->usb,
		 usb_rcvbulkpipe (devolo->usb, 1), devolo->rx_buff,
		 DEVOLO_MAX_MTU, devolo_read_bulk_callback, devolo);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  if ((res = usb_submit_urb (devolo->rx_urb,GFP_ATOMIC)))
#else
  if ((res = usb_submit_urb (devolo->rx_urb)))
#endif
    printk(KERN_ERR "Failed submit rx_urb %d\n", res);
  devolo->flags &= ~DEVOLO_RX_BUSY;
  return;
}


/*
 *   Callback called after packet transmission
 */ 
static void 
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
devolo_write_bulk_callback (struct urb *urb,struct pt_regs * unused)
#else
devolo_write_bulk_callback (struct urb *urb)
#endif
{

  devoloUSB_t *devolo = urb->context;

  if (!devolo || !(devolo->flags & DEVOLO_RUNNING))
    return;

  if (!netif_device_present (devolo->net))
    return;

  devolo->net->trans_start = jiffies;
  /*
   *   Restart packet queue
   */
  netif_wake_queue (devolo->net);

}


/*
 *   Timeout function
 */
static void
devolo_tx_timeout (struct net_device *net)
{
  devoloUSB_t * devolo = net->priv;
  if (!devolo)
    return;
#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)) && (LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,14)))
  devolo->tx_urb->transfer_flags |= URB_ASYNC_UNLINK;
#elif (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
  devolo->tx_urb->transfer_flags |= USB_ASYNC_UNLINK;
#endif
  usb_unlink_urb (devolo->tx_urb);

  /*
   *  Update stats
   */
  devolo->stats.tx_errors++;
}


/*
 *  Function called when kernel need to transmit a packet
 */
static int
devolo_start_xmit (struct sk_buff *skb, struct net_device *net)
{

  devoloUSB_t *devolo = net->priv;
  int count = skb->len+2;
  int res;
  __u16 l16 = skb->len;

  if (!(count % 64)){
      count++;
      l16++;
    }

  if (count < 64){
      count=65;
      l16=63;
    }

  /*
   *   Stop packet queue
   */
  netif_stop_queue (net);

  ((unsigned char *) devolo->tx_buff)[0] = l16 & 0xFF;
  ((unsigned char *) devolo->tx_buff)[1] = (l16 & 0x700) >> 8;
  memcpy (devolo->tx_buff+2, skb->data, skb->len);
 
  FILL_BULK_URB (devolo->tx_urb, devolo->usb,
		 usb_sndbulkpipe (devolo->usb, 2), devolo->tx_buff,
		 count, devolo_write_bulk_callback, devolo);

  /*
   *  Submit urb and update stats
   */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  if ((res = usb_submit_urb (devolo->tx_urb,GFP_ATOMIC))){
#else
  if ((res = usb_submit_urb (devolo->tx_urb))){
#endif
    warn("failed tx_urb %d",res);
    switch (res){
    case -EPIPE:
      /* stall or disconnect... cleanup should 
	 already have been scheduled           */
      break;
    case -ENODEV:
      /* disconnect() upcoming */
      break;
    default:
      devolo->stats.tx_errors++;
      netif_start_queue (net);
    }
  }else{
    devolo->stats.tx_packets++;
    devolo->stats.tx_bytes += skb->len;
    net->trans_start = jiffies;
  }

  dev_kfree_skb (skb);

  return 0;
}


/*
 *   Function that manage stats of the device
 */
static struct net_device_stats *
devolo_netdev_stats (struct net_device *dev)
{
  return &((devoloUSB_t *) dev->priv)->stats;
}


/*
 *   Function that convert hex value to decimal value
 */
static inline unsigned char hex2dec( unsigned char digit )
{
  if ( (digit >= '0') && (digit <= '9') )	
    return (digit - '0');
  
  if ( (digit >= 'a') && (digit <= 'f') )	
    return (digit - 'a' + 10);
  
  if ( (digit >= 'A') && (digit <= 'F') )	
    return (digit - 'A' + 10);
  
  return 16;
}


/*
 *   Function that fill MAC address 
 */
static void set_ethernet_addr( devoloUSB_t *devolo)
{
  int i;
  int len;
  unsigned char buffer[13];
  unsigned char	mac_addr[6];

  len = usb_string(devolo->usb, 3, buffer, 13);
	
  if (len != 12) {
    printk(KERN_ERR "Attempting to get MAC address returned %d bytes\n", len);
    return;
  }
	
  /* 
   *  Fill in the mac_addr 
   */
  for (i = 0; i < 6; i++){
    mac_addr[i] = ( hex2dec( buffer[2 * i] ) << 4 ) + hex2dec( buffer[2 * i + 1] );
  }
	
  /* 
   *  Now copy into the kernel's network driver. 
   */
  memcpy(devolo->net->dev_addr, mac_addr, SIZEOF_MAC);

}

/*
 *  Open function called every time you plug a device
 */ 
static int
devolo_open (struct net_device *net)
{
  devoloUSB_t *devolo = (devoloUSB_t *) net->priv;
  int res;


  // set_ethernet_addr(devolo);

  devolo->flags &= ~DEVOLO_RX_BUSY;
  FILL_BULK_URB (devolo->rx_urb, devolo->usb,
		 usb_rcvbulkpipe (devolo->usb, 1),
		 devolo->rx_buff, DEVOLO_MAX_MTU,
		 devolo_read_bulk_callback,devolo); 

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  if ((res = usb_submit_urb (devolo->rx_urb,GFP_KERNEL/*GFP_ATOMIC??*/)))
#else
  if ((res = usb_submit_urb (devolo->rx_urb)))
#endif
    {
      printk(KERN_ERR "Failed rx_urb %d\n", res);
    }

  netif_start_queue (net);
  devolo->flags |= DEVOLO_RUNNING;

  return 0;
}


/*
 * Close function called every time you unplug a device
 */
static int 
devolo_close (struct net_device *net)
{
  devoloUSB_t * devolo = net->priv;


  devolo->flags &= ~DEVOLO_RUNNING;
  netif_stop_queue (net);

  usb_unlink_urb (devolo->rx_urb);
  usb_unlink_urb (devolo->tx_urb);
  usb_unlink_urb (devolo->ctrl_urb);

  return 0;
}


/*
 *  Manager for ethernet ioctl
 */
static int
netdev_ethtool_ioctl (struct net_device *dev, void *useraddr)
{
  u32 ethcmd;

  if (copy_from_user (&ethcmd, useraddr, sizeof (ethcmd)))
    return -EFAULT;
  switch (ethcmd)
    {
    case ETHTOOL_GDRVINFO:
      {
	struct ethtool_drvinfo info = { ETHTOOL_GDRVINFO };
	strncpy (info.driver, "devolo", sizeof (info.driver) - 1);
	if (copy_to_user (useraddr, &info, sizeof (info)))
	  return -EFAULT;
	return 0;
      }
    }
  return -EOPNOTSUPP;
}


static int read_mii_word( devoloUSB_t *devolo, __u8 phy, __u8 indx, __u16 *regd ){
  return 0;
}

static int write_mii_word( devoloUSB_t *devolo, __u8 phy, __u8 indx, __u16 regd ){
  return 0;
}


/*
 *  Function that manage standard ioctl system calls
 */
static int
devolo_ioctl (struct net_device *net, struct ifreq *rq, int cmd)
{
  __u16 *data = (__u16 *) & rq->ifr_data;
  devoloUSB_t *devolo = net->priv;

  switch (cmd)
    {
    case SIOCETHTOOL:
      return netdev_ethtool_ioctl (net, (void *) rq->ifr_data);
    case SIOCDEVPRIVATE:
      data[0] = devolo->phy;
    case SIOCDEVPRIVATE + 1:
      read_mii_word (devolo, data[0], data[1] & 0x1f, &data[3]);
      return 0;
    case SIOCDEVPRIVATE + 2:
      if (!capable (CAP_NET_ADMIN))
	return -EPERM;
      write_mii_word (devolo, devolo->phy, data[1] & 0x1f, data[2]);
      return 0;
    default:
      return -EOPNOTSUPP;
    }
}


/*
 *   Function for multicast set
 */ 
static void
devolo_set_multicast (struct net_device *net)
{
  devoloUSB_t *devolo = net->priv;

  /*
   *   Stop packet queue
   */
  netif_stop_queue (net);

  if (net->flags & IFF_PROMISC)
    printk(KERN_INFO "%s: Promiscuous mode enabled\n", net->name);
  
  else if ((net->mc_count > multicast_filter_limit) ||
	   (net->flags & IFF_ALLMULTI))
    printk(KERN_INFO "%s set allmulti\n", net->name);
  
  devolo->flags |= ETH_REGS_CHANGE;
  ctrl_callback (devolo->ctrl_urb);

  /*
   *   Restart packet queue
   */
  netif_wake_queue (net);
  
}


/*
 *   Function that allocate, initialize
 *   and register a new device
 */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static int 
devolo_probe (struct usb_interface* intf,
	      const struct usb_device_id *id)
{
  struct usb_device *dev=interface_to_usbdev(intf);
#else 
static void *
  devolo_probe (struct usb_device *dev, unsigned int ifnum,
	      const struct usb_device_id *id)
{
#endif
  struct net_device *net;
  devoloUSB_t *devolo;
  int dev_index = 0;
  int err;
  
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  if (usb_reset_configuration (dev))
#else
  if (usb_set_configuration (dev, dev->config[0].bConfigurationValue))
#endif
    {
      printk(KERN_ERR "usb_set_configuration failed\n");
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
      return -ENODEV;
#else
      return NULL;
#endif
    }

  if (!(devolo = kmalloc (sizeof (struct devoloUSB), GFP_KERNEL)))
    {
      printk(KERN_ERR "Out of memory allocating device structure\n");
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
      return -ENOMEM;
#else
      return NULL;
#endif
    }

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  usb_get_dev(dev);
#else
  usb_inc_dev_use (dev);
#endif
  memset (devolo, 0, sizeof (struct devoloUSB));
  devolo->dev_index = dev_index;
  init_waitqueue_head (&devolo->ctrl_wait);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))

  devolo->ctrl_urb=usb_alloc_urb(0,GFP_KERNEL);
  if(!devolo->ctrl_urb) return -ENOMEM;
  devolo->rx_urb=usb_alloc_urb(0,GFP_KERNEL);
  if(!devolo->rx_urb) {
    usb_free_urb(devolo->ctrl_urb);
    return -ENOMEM;
  }
  devolo->tx_urb=usb_alloc_urb(0,GFP_KERNEL);
  if(!devolo->tx_urb) {
    usb_free_urb(devolo->ctrl_urb);
    usb_free_urb(devolo->rx_urb);
    return -ENOMEM;
  }
  devolo->intr_urb=usb_alloc_urb(0,GFP_KERNEL);
  if(!devolo->intr_urb) {
    usb_free_urb(devolo->ctrl_urb);
    usb_free_urb(devolo->rx_urb);
    usb_free_urb(devolo->tx_urb);
    return -ENOMEM;
  }
#else
  devolo->ctrl_urb=usb_alloc_urb(0);
  devolo->rx_urb=usb_alloc_urb(0);
  devolo->tx_urb=usb_alloc_urb(0);
  devolo->intr_urb=usb_alloc_urb(0);
#endif


  /*
   *	Allocate a device
   */
  net = alloc_etherdev(NET_SIZE_OF_PRIV);
  if(net==NULL)
    {
      usb_free_urb(devolo->ctrl_urb);
      usb_free_urb(devolo->rx_urb);
      usb_free_urb(devolo->tx_urb);
      usb_free_urb(devolo->intr_urb);
      kfree (devolo);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
      return -ENOMEM;
#else
      return NULL;
#endif
    }

  /*
   *  Allocate a name
   */
  strcpy(net->name, DEVICE_NAME_MASK);
  if (dev_alloc_name(net, DEVICE_NAME_MASK)<0) {
    usb_free_urb(devolo->ctrl_urb);
    usb_free_urb(devolo->rx_urb);
    usb_free_urb(devolo->tx_urb);
    usb_free_urb(devolo->intr_urb);
    kfree(net);
    kfree (devolo);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
      return -ENOMEM;
#else
      return NULL;
#endif
  }

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
  init_etherdev (net, NET_SIZE_OF_PRIV);
#endif
  /*
   *  Register the device
   */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
  rtnl_lock();
  err = register_netdevice(net);
  rtnl_unlock();
  if (err < 0)
    {
      usb_free_urb(devolo->ctrl_urb);
      usb_free_urb(devolo->rx_urb);
      usb_free_urb(devolo->tx_urb);
      usb_free_urb(devolo->intr_urb);
      kfree(net);
      kfree(devolo);
      net = NULL;
      return NULL;
    }
#endif

  /*
   *  Implement operations needed by the device
   */
  ether_setup (net);
  devolo->usb = dev;
  devolo->net = net;
  SET_MODULE_OWNER (net);
  net->priv = devolo;
  net->open = devolo_open;
  net->stop = devolo_close;
  net->watchdog_timeo = DEVOLO_TX_TIMEOUT;
  net->tx_timeout = devolo_tx_timeout;
  net->do_ioctl = devolo_ioctl;
  net->hard_start_xmit = devolo_start_xmit;
  net->set_multicast_list = devolo_set_multicast;
  net->get_stats = devolo_netdev_stats;
  net->mtu = DEVOLO_MTU;
  devolo->phy = 1;

  set_ethernet_addr(devolo);

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  usb_set_intfdata(intf, devolo);
  SET_NETDEV_DEV(net, &intf->dev);
  rtnl_lock();
  err = register_netdevice(net);
  rtnl_unlock();
  return 0;
#else
  return devolo;
#endif
}


/*
 *   Function that free all allocated resource
 */
 static void
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
devolo_disconnect (struct usb_interface* intf){
   struct devoloUSB *devoloUSB = usb_get_intfdata(intf);
   struct usb_device *dev=interface_to_usbdev(intf);
#else
devolo_disconnect (struct usb_device *dev, void *ptr){
  struct devoloUSB *devoloUSB = ptr;
#endif
  if (!devoloUSB)
    {
      printk(KERN_ERR "Unregistering non-existant device\n");
      return;
    }

  devoloUSB->flags |= DEVOLO_UNPLUG;
  unregister_netdev (devoloUSB->net);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
  usb_put_dev(dev);
#else
  usb_dec_dev_use (dev);
#endif
  usb_free_urb(devoloUSB->rx_urb);
  usb_free_urb(devoloUSB->tx_urb);
  usb_free_urb(devoloUSB->intr_urb);
  usb_free_urb(devoloUSB->ctrl_urb);
  kfree (devoloUSB->net);
  kfree (devoloUSB);
  devoloUSB = NULL;

}

/*
 *   Structure needed to register our objects
 *   and find our hardware device
 */
static struct usb_driver devolo_driver = {
  name:"devolo",
  probe:devolo_probe,
  disconnect:devolo_disconnect,
  id_table:plUsb_ids,
};

int __init
devolo_init (void)
{
  printk (KERN_INFO DRIVER_DESC ":" DRIVER_VERSION "\n");
/*   char usbmgr[9]={0}; */

#ifdef USBMGR

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))

#define usbmgr USBMGR"_hcd"
/*   sprintf(usbmgr,"%s_hcd",USBMGR); */
#else
#define usbmgr "usb-"USBMGR
/*   sprintf(usbmgr,"usb-%s",USBMGR); */
  
#endif

  request_module(usbmgr);
#endif
  return usb_register (&devolo_driver);
}

void __exit
devolo_exit (void)
{
  usb_deregister (&devolo_driver);
}


module_init (devolo_init);
module_exit (devolo_exit);
