/* 
   devolo_usb.h: dLAN USB driver for linux. 

   Only for use with dLAN adaptors from devolo.
   Copyright 2007 devolo AG		    
 
 */

#include <linux/version.h>

#define	DLAN_ALIGN(x)		x __attribute__((aligned(L1_CACHE_BYTES)))
#define SIZEOF_MAC 6
#define DEVICE_NAME_MASK "dlanusb%d"
#define NET_SIZE_OF_PRIV 0 

/*
 * Driver Version Information
 */
#define DRIVER_VERSION "v4"
#define DRIVER_AUTHOR "devolo AG"
#define DRIVER_DESC "Driver for dLAN USB"

#define	DEVOLO_MTU		1500
#define DEVOLO_MAX_MTU          1536
#define USB_INTELLON_VENDOR_ID	0x09e1
#define USB_INTELLON_PRODUCT_ID	0x5121
#define DEVOLO_BUFFER_SIZE	1536 
#define DEVOLO_INTERRUPT_INTERVAL 200   

/*
 * Interrupt pipe index  
 */
#define DEVOLO_INTERRUPT_PIPE	   0x01
#define DEVOLO_BULKOUT_PIPE	   0x02
#define DEVOLO_BULKIN_PIPE	   0x03

/*
 * Endpoints max packet dimensions 
 */
#define DEVOLO_CONTROL_BUFSIZE	   0x20 /** endpoint 0 max packet dimension **/
#define DEVOLO_DATAIN_BUFSIZE	   0x40 /** endpoint 1 max packet dimension **/
#define DEVOLO_DATAOUT_BUFSIZE	   0x40 /** endpoint 2 max packet dimension **/
#define DEVOLO_INTERRUPT_BUFSIZE   0x08 /** endpoint 3 max packet dimension **/

/* 
 * Driver Flags, constants and struct 
 */
#define	DEVOLO_PRESENT	(1<<0)
#define	DEVOLO_RUNNING	(1<<1)
#define	DEVOLO_TX_BUSY	(1<<2)
#define	DEVOLO_RX_BUSY	(1<<3)
#define	DEVOLO_UNPLUG	(1<<4)
#define ETH_REGS_CHANGE (1<<5)
#define	DEVOLO_TX_TIMEOUT	(HZ*10)




#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

#define FILL_BULK_URB(URB,DEV,PIPE,TRANSFER_BUFFER,BUFFER_LENGTH,COMPLETE,CONTEXT) usb_fill_bulk_urb(URB,DEV,PIPE,TRANSFER_BUFFER,BUFFER_LENGTH,COMPLETE,CONTEXT)

#endif // LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)


/* 
 * This structure contains all the global objects the driver
 * needs to discribe the device and its status 
 */
typedef struct devoloUSB {
	struct usb_device	*usb;
	struct net_device	*net;
	struct net_device_stats	stats;
	unsigned		flags; 
	__u16                   mode_flags;
	unsigned		features;
	int			dev_index;
	int			intr_interval;
	struct urb		*ctrl_urb, *rx_urb, *tx_urb, *intr_urb;
	wait_queue_head_t	ctrl_wait;
	struct semaphore	ctrl_sem;
	unsigned char		DLAN_ALIGN(rx_buff[DEVOLO_MAX_MTU]);
	unsigned char		DLAN_ALIGN(tx_buff[DEVOLO_MAX_MTU]);
	unsigned char		DLAN_ALIGN(intr_buff[DEVOLO_INTERRUPT_BUFSIZE]);
	__u8			eth_regs[4];
	__u8			phy;
	__u8			gpio_res;
} devoloUSB_t;


/* 
 * CDC Control Request macros and struct 
 */
#define CDC_CLASS_IN_REQUEST		  USB_DIR_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE 
#define CDC_CLASS_OUT_REQUEST		  USB_DIR_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE 

#define CDC_SEND_ENCAPSULATED_COMMAND	  0x00 /* Type */
#define CDC_GET_ENCAPSULATED_RESPONSE	  0x01 /* Type */
#define	CDC_SET_ETHERNET_PACKET_FILTER    0x43 /* Type */
#define CDC_GET_ETHERNET_STATISTICS	  0x44 /* Type */
#define CDC_FEATURE_SELECTOR_RCV_OVERRUN  0x19 /* Value field of CDC_GET_ETHERNET_STATISTICS command */

/* 
 * Value field of CDC_SET_ETHERNET_PACKET_FILTER command 
 */
#define MODE_FLAG_PROMISCUOUS           (1<<0)
#define MODE_FLAG_ALL_MULTICAST         (1<<1)
#define MODE_FLAG_DIRECTED              (1<<2)
#define MODE_FLAG_BROADCAST             (1<<3)
#define MODE_FLAG_MULTICAST             (1<<4)

/*
 * FunctionCode value used in CDC_SEND_ENCAPSULATED_COMMAND 
 * and CDC_GET_ENCAPSULATED_RESPONSE 
 */
#define USBIF_FUNC_SET_PHY_REG            0x01
#define USBIF_FUNC_GET_PHY_REG            0x02
#define USBIF_FUNC_SET_USB_REG            0x03
#define USBIF_FUNC_GET_USB_REG            0x04
#define USBIF_FUNC_VERSION_INFO           0x05

struct USBIF_HW_VERSION_INFO_IN {
	unsigned char FunctionCode; 
	unsigned short BootLoaderVersion;  
	unsigned short BoardVersion;  
	unsigned short BoardSubVersion; 
	unsigned short ProductionYear;
	unsigned char ProductionWeek;   
	unsigned char MAC_Address[6];
} __attribute__ ((packed));

struct USBIF_HW_VERSION_INFO_OUT {
	unsigned char  FunctionCode;
}  __attribute__ ((packed));
 
struct USBIF_SET_PHY_REG_OUT {
	unsigned char FunctionCode; 
	unsigned char RegAddress; 
	unsigned short Data;  
} __attribute__ ((packed));

struct USBIF_GET_PHY_REG_OUT {
	unsigned char FunctionCode; 
	unsigned char RegAddress;  
} __attribute__ ((packed));

struct USBIF_GET_PHY_REG_IN {
	unsigned char FunctionCode; 
	unsigned short Data;  
} __attribute__ ((packed));

struct USBIF_SET_USB_REG_OUT {
	unsigned char FunctionCode; 
	unsigned char RegAddress; 
	unsigned short Data;  
} __attribute__ ((packed));

struct USBIF_GET_USB_REG_OUT {
	unsigned char FunctionCode; 
	unsigned char RegAddress; 
}  __attribute__ ((packed));

struct USBIF_GET_USB_REG_IN {
	unsigned char FunctionCode; 
	unsigned short Data;  
} __attribute__ ((packed));
