
/*
   dlanconfig.c: dlanconfig software for HomePlug devices for linux

   Copyrighted 2007 by devolo AG

   Redistribution in source form, with or without modification,
   are NOT permitted.
   This program uses explicitly confidential information
   about HomePlug standard.
*/

#include "md5.h"
#include "dlanconfig.h"
#include "sha256.h"

#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <stdio.h>
#include <sys/uio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h>
#include <unistd.h>
#include <string.h>
#include <sys/mman.h>
#include <linux/if_packet.h>
#include <ctype.h>
#include <wait.h>

/*
 *  Defines
 */
#define LOCAL  '1'
#define REMOTE '2'
#define POLL '3'
#define EXIT '4'
#define GETLOCALMAC  '5'

#define EXITstr   "Exit"
#define POLLstr   "List remote devices"
#define LOCALstr  "Set local network password"
#define REMOTEstr "Set remote network password"
#define PRINTOPT(a,b)  printf("\t%c) %s\n",a,b)
#define DOWN 1
#define UP   0

#define KEEP_MME_SETCHAR
#define KEEP_ENABLE_BACKPRESSURE
// number of retry to get localMAC
#define MAXRETRY 5

/*
 *  Global variables
 */
static char localifname[MAX_INAME_LEN];
static unsigned char localMAC[MACSIZE];
static char localifstatus = UP;
static devolo_table_t p_devTable;
static char viper_setlocalnek_succeed;
static char hp1device_localnek_setting;
static char av200_setlocalnek_succeed;
static AV200_table_t p_avTable;

static int AV200_isoldversion = FALSE;

/*
 * Functions prototypes
 *
 */
int InitIf(char *name, unsigned char *macAddress);
void Usage(char *progname);
int SendPacket(char *iname,unsigned char addr[],void *buffer,size_t len);
int WaitForNBytes(int fileDesc, int expectedBytes, unsigned char* buf, struct timeval timeValue, int extraTimeSeconds);
int GetNextPacket(int fileDesc, void *strPacket);
pid_t ForkProcesses(char action, char *data, unsigned char *newNek, unsigned char *remoteDek);
void CloseAll(int fileDesc, pid_t pid);
void IfDown(char *name);
int SendPacketAV200(char *iname,unsigned char addr[],void *buffer,size_t len);
void GetAV200VendorSpecific(void *packet);
int SendViperPacketMME_DDR(char *iname);
void GetLocalParExtResp(void* packet);
int SendHP1ParamPacket(char *iname);
void GetPrivateVendorSpecific(void *packet);
int SendStatRequest(char *iname);
void GetUpdateDeviceTable(unsigned char *mac, unsigned char deviceType);
void GetUpdateDeviceTableAV200(unsigned char *mac, unsigned char tei, unsigned char *firstBridgedMac, unsigned char txRate, unsigned char rxRate);
int SendNetStatsRequest(char *iname);
int SendViperPacketMME_QIR(char *iname);	
int SendSetRemoteViperNek(char *iname, unsigned char* newNek, unsigned char * remoteDek);
int SendSetRemoteHP1Nek(char *iname, unsigned char* newNek, unsigned char* remoteDek);
int SendSetLocalNek(char *iname, unsigned char *newNek);
void GetUpdatePhyDataRates(void *packet);
int LocalBridgeAlreadyFound();
void ClearRemoteDevoloTable();
int LocalBridgeAlreadyFoundAV200();
void ClearRemoteAV200Table();
int LookDevicesAV200(char *iname, int pipe);
int SetLocalNEKAV200(char *iname, int pipe, unsigned char *newNek);
int SetRemoteNEKAV200(char *iname, int pipe, unsigned char *newNek, unsigned char *deviceDek);
int SendNetworkInfoAV200(char *iname);
int SendLocalAV200SetKeyRequest(char *iname, unsigned char *newNek);
int SendRemoteAV200SetKeyRequest(char *iname, unsigned char *newNek, unsigned char *deviceDek);
int SendAV200GetVersionRequest(char *iname);
int SendAV200NetworkInfoRequest(char *iname);
void GetAV200NetworkInfo(void *packet);
void GetAV200SetKeyCnf(void *packet);
int LookDevices(char *iname, int pipe);
int GetLocalMAC(char *iname, int pipe);
int SetLocalNEK(char *iname, int pipe, unsigned char *newNek);
int SetRemoteNEK(char *iname, int pipe, unsigned char *newNek, unsigned char *deviceDek);
void PrintNode(char *iname);
void PrintNodeAV200(char *iname);
int API_NetwPoll(char *iname);
int ListenForPackets(int fileDescr,int maxNumber,int maxTimeS);
inline void PrintMAC(unsigned char *mac_address);
inline int IsSameMAC(unsigned char* mac1,unsigned char* mac2);
inline int IsAlmostSameMAC(const void* mac1, const void* mac2);
inline double Viper_DisplayRate(double dBytesPer336usBlock);
inline double HP1_DisplayRate(double dBytes40);
inline void PrintMenu();



/*
 * Usage
 */
void Usage(char *progname){

  	printf("Usage: %s interface\n",progname);
  	printf("Example: %s dlanusb0\n\n",progname);
}

/*
 *	Print a formatted version of MAC address
 */
inline void PrintMAC(unsigned char *mac_address) 
{ 
	unsigned char* mac;
	for(mac = mac_address; mac < mac_address + MACSIZE - 1; mac++)
		printf("%02X:", *mac);
	printf("%02X", *mac);
	fflush(stdout);
}


/*
 *	Function that print the list of device in the lan
 */
void PrintNode(char *iname)
{
	unsigned int i,j;
	unsigned int hp1_counter, viper_counter;
	double aggRate_hp1, aggRate_viper;
	j = 1;
	hp1_counter = 0;
	viper_counter = 0;
	aggRate_hp1 = 0.0;
	aggRate_viper = 0.0;

	printf("\n");
	if (p_devTable.devCounter <= 0)
	{
		printf("\tNo remote devices found!\n");
		return;
	}
	
	for (i = 0; i < p_devTable.devCounter; i++)
	{
		if (p_devTable.list[i].devType == ISVIPER)
		{
			viper_counter++;
			aggRate_viper += p_devTable.list[i].rate;
		}else
		{
			hp1_counter++;
			aggRate_hp1 += p_devTable.list[i].rate;
		}
	}
	if (hp1_counter)
		aggRate_hp1 /= hp1_counter;

	if (viper_counter)
		aggRate_viper /= viper_counter;

	printf("\n\tRemote devices:\n");
	printf("\tNum.\tMAC address	\tMbps \tCache \tHighSpeed\n");
	
	for (i = 0; i < p_devTable.devCounter; i++)
	{
		printf("\t%d\t", j++);
		PrintMAC(p_devTable.list[i].MAC);
		if (p_devTable.list[i].rate > 0.0)
			printf("\t%1.3f", p_devTable.list[i].rate);
		else
			printf("\tN/A");
		if (p_devTable.list[i].toggled)
			printf("\t  *");
		else
			printf("\t   ");
		if (p_devTable.list[i].devType == ISVIPER)
			printf("\t  #");
		else
			printf("\t   ");
		printf("\n");
	}	
	printf("\n");
	
	if (hp1_counter)
		printf("\tPhysical average data rate (Normal devices      ) = %1.3f Mbps\n", aggRate_hp1);
	if (viper_counter)
		printf("\tPhysical average data rate (Highspeed devices   ) = %1.3f Mbps\n", aggRate_viper);
	fflush(stdout);
}


/*
 *	Print the list of AV200 devices
 */
void PrintNodeAV200(char *iname)
{
	int i;
	
	if ( p_avTable.devCounter <= 0)
	{
		printf("\n\tNo remote devices found!\n");
		return;
	}
	printf("\n\t\t Remote devices:\n\n");
	printf("\tNum.\tMAC address	\tTx\tRx\n");
	for (i = 0; i < p_avTable.devCounter; i++)
	{
		printf("\t%d\t", (i+1));
		PrintMAC(p_avTable.list[i].MAC);
		if ( (p_avTable.list[i].avgTxPhyRateMbps == 0) || (p_avTable.list[i].avgRxPhyRateMbps == 0) )
		{
			printf("\tidle\tidle ");
		} else {
			printf("\t%4.2f", ((double)p_avTable.list[i].avgTxPhyRateMbps * 21.0/16.0));
			printf("\t%4.2f", ((double)p_avTable.list[i].avgRxPhyRateMbps * 21.0/16.0));
		}
		printf("\n");
	}
	fflush(stdout);
}
/*
 *	Print the main menu
 */
inline void PrintMenu()
{
	printf("\n\t==================================\n");
	printf("\n\t\tdLAN-Config main menu\n");
	printf("\n\t==================================\n\n");
	PRINTOPT(LOCAL, LOCALstr);
	PRINTOPT(REMOTE, REMOTEstr);
	PRINTOPT(POLL, POLLstr);
	PRINTOPT(EXIT, EXITstr);
	printf("\n\tChoose a command number: ");
}
/*
 *	Check if two MAC address are equal
 */
inline int IsSameMAC(unsigned char* mac1,unsigned char* mac2)
{
	int i;
	for(i = 0; i < MACSIZE; i++)
		if(mac1[i] != mac2[i])
			return 0;
	return 1;
}

inline int IsAlmostSameMAC(const void* mac1, const void* mac2)
{
	
	char* macA;
	char* macB;
	int flag = 0;
	macA = (char*) mac1;
	macB = (char*) mac2;
	
	flag = ((macA[0] | 0x01) == (macB[0] | 0x01));
	return (flag && macA[1] == macB[1] && macA[2] == macB[2] && macA[3] == macB[3] && macA[4] == macB[4] && macA[5] == macB[5]);

}

/*
 *	Calculate and returns the speed of Viper
 */
inline double Viper_DisplayRate(double dBytesPer336usBlock)
{
	static const double dMaxTrueRate = 6.0;
	static const double dMaxHp1Rate = ((519.0 * 8.0) / (40.0 * 8.4));	//12.35714286
	static const double dMaxTurboRate = ((2812.0 * 8.0) / (40.0 * 8.4));	//66.95238095

	double dCalculatePhyRate = dBytesPer336usBlock * 8.0 / (40.0 * 8.4);
	
	if (dCalculatePhyRate <= dMaxTrueRate)
		return dCalculatePhyRate;
	else if ((dCalculatePhyRate > dMaxTrueRate) && (dCalculatePhyRate <= dMaxHp1Rate))
		return (((dCalculatePhyRate - dMaxTrueRate) / (dMaxHp1Rate - dMaxTrueRate)) * (14 - 6)) + 6;
	else
		return (((dCalculatePhyRate - dMaxHp1Rate) / (dMaxTurboRate - dMaxHp1Rate)) * (85 - 14)) + 14;
}

/*
 *	Calculate and returns the speed of HP1
 */
inline double HP1_DisplayRate(double dBytes40)
{
	static const double dConstantM = ((588 - 38.0) / 481.0);
	static const double dConstantB = (14.0 - dConstantM * (519.0 / 42.0));

	return (dConstantM * (dBytes40 / 42.0)) + dConstantB;
}

/*
 *	Clear the table of remote devolo devices
 */
void ClearRemoteDevoloTable()
{
	int i;
	for (i = 0; i < p_devTable.devCounter; i++)
	{
		memset(p_devTable.list[i].MAC, 0, MACSIZE);
		memset(p_devTable.list[i].DEK, 0, DECKSIZE);
		memset(p_devTable.list[i].NEK, 0, DECKSIZE);
		p_devTable.list[i].rate = 0.0;
		p_devTable.list[i].toggled = 0;
		p_devTable.list[i].devType = 0;
		p_devTable.list[i].isHere = 0;
	}
	p_devTable.devCounter = 0;
}

/*
 *	Returns 1 if local bridge have been found
 */
int LocalBridgeAlreadyFound()
{
	unsigned char nullMAC[MACSIZE];

	memset( nullMAC, 0, MACSIZE);
	return !IsSameMAC(nullMAC, p_devTable.locBridge.MAC);
}

/*
 *	Returns 1 if local AV200 bridge have been found
 */
int LocalBridgeAlreadyFoundAV200()
{
	unsigned char nullMAC[MACSIZE];

	memset( nullMAC, 0, MACSIZE);
	return !IsSameMAC(nullMAC, p_avTable.localBridge.MAC);
}

/*
 *	Clear the table of remote AV200 devices
 */
void ClearRemoteAV200Table()
{
	int i;

	for (i = 0; i < p_avTable.devCounter; i++)
	{
		memset(p_avTable.list[i].MAC, 0, MACSIZE);
		p_avTable.list[i].TEI = 0;
		memset(p_avTable.list[i].FirstBridgedMAC, 0, MACSIZE);
		p_avTable.list[i].avgTxPhyRateMbps = 0;
		p_avTable.list[i].avgRxPhyRateMbps = 0;
		p_avTable.list[i].isHere = 0;
	}
	p_avTable.devCounter = 0;
}

/*
 * Checks if the iff_up flags is set and bring the interface up if necessary. Sets interface MAC.
 * param in 	name : interface name
 * param out	macAddress : MAC address
 * return 	1 on error, 0 on success
 */
int InitIf(char *name, unsigned char *macAddress)
{
	int sock;
	struct ifreq ifr;
	// Controls that macAddress variable is valid
	if (!macAddress)
	{
		printf("Internal error. %s\n", __FUNCTION__);
		return 1;
	}
	// Opens a socket
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	// If sock == -1 the socket cannot be open
	if (sock == -1)
	{
		perror("\n Error opening socket. ");
		return 1;
	}
	// Sets to 0 ifr variable
	memset(&ifr, 0, sizeof(struct ifreq));
	// Copy the name passed for argument int the interface request variable
	strncpy(ifr.ifr_name, name, IF_NAMESIZE);

	if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0)
	{
		sleep(1); // Necessary to the kernel
		if (ioctl(sock, SIOCGIFFLAGS, &ifr) < 0)
		{
			perror(NULL);
			close(sock);
			return 1;
		}
	}
	// Controls if the interface is down
	if ((ifr.ifr_flags & IFF_UP) == 0)
	{
		localifstatus = DOWN;
		printf("\n\tInterface %s is down. Bringing up..", name);
		fflush(stdout);
		// Modify the flag
		ifr.ifr_flags |= IFF_UP;
		// Send the set command of the interface flags to change it
		if (ioctl(sock, SIOCSIFFLAGS, &ifr) < 0)
		{
			perror("\nError bringing up. ");
			close(sock);
			return 1;
		}
		printf(".successfull!\n");
	}
	// Get the MAC address
	if (ioctl(sock, SIOCGIFHWADDR, &ifr) == -1)
	{
		perror("\nError getting MAC address. ");
		close(sock);
		return 1;
	}
	// Copies the MAC into the macAddress variable
	memcpy(macAddress, ifr.ifr_hwaddr.sa_data, MACSIZE);
	close(sock);
	return 0;
}
/*
 *	Brings the interface iname down
 */
void IfDown(char *name)
{
	int fileDesc;
	struct ifreq ifr;
	
	fileDesc = socket(AF_INET, SOCK_DGRAM, 0);
	printf("\n\tBringing interface %s down...\n\n", name);
	if (fileDesc == -1)
	{
		perror("Error opening socket");
		return;
	}
	memset(&ifr, 0, sizeof(struct ifreq));
	strncpy(ifr.ifr_name, name, IF_NAMESIZE);
	if (ioctl(fileDesc, SIOCSIFFLAGS, &ifr) < 0)
		perror("Error setting flag");
}



/*====================================================================================================================
 *	Functions for packet's handling
 *
 *====================================================================================================================*/

/*
 *	Wait for a number N of bytes to be read via socket
 */
int WaitForNBytes(int fileDesc, int expectedBytes, unsigned char* buf, struct timeval inputTimeValue, int extraTimeSeconds)
{
	fd_set readfds;
	int arrivedBytes = 0;
	int ready = 0;
	struct timeval firstTimeVal, secondTimeVal;
	
	// Clear readfds
	FD_ZERO(&readfds);
	// Assign fileDesc to the fd_set for reading purpose
	FD_SET(fileDesc, &readfds);
	firstTimeVal.tv_sec = inputTimeValue.tv_sec;
	firstTimeVal.tv_usec = inputTimeValue.tv_usec;
	// Uses the select function to know when the packet arrives
	ready = select(fileDesc + 1, &readfds, NULL, NULL, &firstTimeVal);
	// If arrives a packet reads from fileDesc into the buffer starting at buf
	if (ready)
		arrivedBytes = read(fileDesc, buf, expectedBytes);
	else
		return 0;	
	if (arrivedBytes == expectedBytes)
		return 1;
	// If some bytes are missing
	if (arrivedBytes < expectedBytes)
	{
		// Prepare another fd_set and tries to get the remaining part of the bytes
		FD_ZERO(&readfds);
		FD_SET(fileDesc, &readfds);
		secondTimeVal.tv_sec = 3;
		secondTimeVal.tv_usec = 0;
		ready = select( fileDesc + 1, &readfds, NULL, NULL, &secondTimeVal);
		if (ready)
			arrivedBytes += read(fileDesc, buf, expectedBytes - arrivedBytes);
		if (arrivedBytes == expectedBytes)
		{
			return 1;
		}
		else
		{
			printf("\nError-> WaitForNBytes: no bytes to read\n");
			return (-1);
		}
	}
	printf("\nError-> WaitForNBytes: incomplete reading\n");
	return -1;
}

/*
 *	Get successive packet
 */

int GetNextPacket(int fileDesc, void *strPacket)
{
	struct timeval timeValue;
	int returnValue = 0;
	char strHeader[PACKET_HEADERSIZE] = "";
	int packetLength = 0;
	
	timeValue.tv_sec = SELECT_TIMEOUT_S;
	timeValue.tv_usec = SELECT_TIMEOUT_US;
	// Uses WaitForNBytes to read header
	returnValue = WaitForNBytes(fileDesc, PACKET_HEADERSIZE - 1, (unsigned char*)strHeader, timeValue, EXTRA_WAIT_TIME_S);
	switch (returnValue)
	{
		case -1:
			printf("\n Error-> GetNextPacket: broken protocol (incomplete header)\n");
			return -1;

		case 0:
			return 0;

		case 1: // In this case reads the next bytes that are of the packet
			strHeader[PACKET_HEADERSIZE - 1] = '\0';
			packetLength = atoi(strHeader);
			memset(strPacket, 0, MAX_PACK_LEN);
			returnValue = WaitForNBytes(fileDesc, packetLength, strPacket, timeValue, EXTRA_WAIT_TIME_S);
			if (returnValue == -1)
			{
				printf("\n Error-> GetNextPacket: broken protocol (incomplete packet)\n");
				return (-1);
			}
			if (returnValue == 0)
			{
				printf("\n Error-> GetNextPacket: broken protocol (header present, no packet)\n");
				return (0);
			}
			{
				int counter = 0;
				struct API_ReqNetStatsResp* p;
				p = (struct API_ReqNetStatsResp*)strPacket;
				counter++;
			}
		return 1;
	}
	return -1;
}
/*
 *	Send a packet to the interface iname
 */
int SendPacket(char *iname,unsigned char addr[],void *buffer,size_t len)
{
	int i = 0;
	int returnValue = 0;
	struct sockaddr_ll destAddr;
	struct sockaddr_ll sourceAddr;
	int sock = 0;
	struct ifreq ifr;

	memset(&sourceAddr, 0, sizeof(sourceAddr));
	memset(&destAddr, 0, sizeof(destAddr));
	strcpy(ifr.ifr_name,iname);

	sock = socket(PF_PACKET, SOCK_DGRAM, htons(SOCK_PROTO));
	if ( sock < 0 )
	{
		printf("[ERROR]: SendPacket: socket unable to send packet \n");
		return -1;
	}

	if (ioctl(sock,SIOCGIFINDEX,&ifr))
	{
		printf("[ERROR]: SendPacket: error ioctl SIOCGIFINDEX errno=%d\n",errno);
		close(sock);
	return -1;
	}

	destAddr.sll_family = AF_PACKET;
	destAddr.sll_protocol =htons(ETHERTYPE_INTELLON);
	destAddr.sll_ifindex= ifr.ifr_ifindex;
	destAddr.sll_halen = 6;

	for(i=0;i<MACSIZE;i++)
		destAddr.sll_addr[i]=addr[i];

	sourceAddr.sll_family = PF_PACKET;
	sourceAddr.sll_protocol=htons(SOCK_PROTO);
	sourceAddr.sll_ifindex = ifr.ifr_ifindex;
	returnValue = bind(sock,(struct sockaddr *) &sourceAddr, sizeof(sourceAddr));
	if ( returnValue==-1)
	{
		perror("SendPacket: bind error\n");
	}else
		returnValue = sendto(sock, buffer, len, 0,(struct sockaddr *) &destAddr,sizeof destAddr);

	if ( returnValue==-1)
		perror("SendPacket: sendto error\n");
#ifdef DLANCONFIG_DEBUG
	printf("\n\tSent packet to %2x:%2x:%2x:%2x:%2x:%2x\n", addr[0], addr[1], addr[2], addr[3], addr[4], addr[5]);
#endif
	close(sock);
	return(returnValue);
}

/*
 *	Parser of received packet 
 */
void PacketReceive_callback(unsigned char *packet)
{
	struct MME_PackedHdr* pHdr;
	
	if(packet == NULL)
		return;
	pHdr = (struct MME_PackedHdr*)packet;
#ifdef DLANCONFIG_DEBUG	
	printf("\n\tReceived packet with pHdr->MEHDR:%x\n",pHdr->MEHDR); 
#endif	
	if ((!IsSameMAC(pHdr->DA,localMAC)) || ((pHdr->Ethertype!=htons(ETHERTYPE_INTELLON)) && (pHdr->Ethertype != htons(ETHERTYPE_AV200)))) {
#ifdef DLANCONFIG_DEBUG
		printf("\n\tPacket is not for us or is not ETHERTYPE_INTELLON or ETHERTYPE_DEVOLO\n"); 
#endif
		return;
	}
	if (( pHdr->MCTRL == MME_VERSION_AV200) && (pHdr->MEHDR == 0x01))
	{
		GetAV200VendorSpecific(packet);
		return;
	}
	if (( pHdr->MCTRL == MME_VERSION_AV200) && (pHdr->MEHDR == 0x39))
	{
		GetAV200NetworkInfo(packet);
		return;
	}
	if (( pHdr->MCTRL == MME_VERSION_AV200) && (pHdr->MEHDR == 0x51))
	{
		GetAV200SetKeyCnf(packet);
		return;
	}
	if((pHdr->MEHDR == CODE_ReqStatsResp)) { 
	
		if(viper_setlocalnek_succeed == 0){
			if(!IsSameMAC(pHdr->SA, p_devTable.locBridge.MAC)){
				viper_setlocalnek_succeed=1;
			}
		}
		GetUpdateDeviceTable(pHdr->SA, ISOLDDEVICE);
		return; 
	}
	
	if( (pHdr->MEHDR == CODE_ReqNetStats)){
		GetUpdatePhyDataRates(packet);
		return;
	}
	
	if((pHdr->MEHDR == CODE_SetLocPar) && (pHdr->NETW_CTRL == 0x1)){
		GetLocalParExtResp(packet);
		return;
	}
	
	if( (pHdr->MEHDR == CODE_PrivateVendorSpec) ){    
		GetPrivateVendorSpecific(packet);
		return;
	}
	
	
	if( (pHdr->MEHDR == CODE_HPSetNEKResp) ){    
		viper_setlocalnek_succeed=1;
		return;
	}
	printf("\n\t\tPacket Ignored\n"); 
	
  
}
/*
 *   Listen until new packets arrive
 *   return number of received packet, -1 on error
 */
int ListenForPackets(int fileDescr,int maxNumber,int maxTimeS)
{
	int receivedCount = 0;
  	int timeoutCount = 0;
  	int timeoutN = 0;
  	int returnValue = 0;
  	unsigned char buffer[MAX_PACK_LEN];  

  	timeoutN = (1000000*maxTimeS)/SELECT_TIMEOUT_US;
  	while(receivedCount < maxNumber && timeoutCount < timeoutN) {
    		returnValue = GetNextPacket(fileDescr, buffer);
    		switch(returnValue){
    			case -1:
      				printf("[ERROR]: ListenForPackets: get next packet = %d\n",returnValue );   
      				return -1;
    			case 0:
      				timeoutCount++;  
      				break;
    			case 1:
      				PacketReceive_callback(buffer); 
      				receivedCount++;    
      				break;
    		}
  	}
  	return(receivedCount);
}


/*
 * Check Versioning Since MAC-1-5
 */
#define OLDVERSION 0x104
int IfOldVersion(unsigned char *data){
  int min; 
  int maj; 
  int i;
  int version = OLDVERSION;
  char temp[]="INT6000-MAC-";
  char * app=NULL;
  
  for(i=0;i<53;i++){
    if(data[i] == 0) data[i] = 'x';
  }
  data[52]=0;
  
  app=strstr(data,temp);
  app+=strlen(temp);
  maj = atoi(app);
  app=strchr(app,'-');
  app++;
  min = atoi(app);
  return (((maj<<8)|min) <= version);
}


/*
 *	Gets info about AV200 stored in packet
 */
void GetAV200VendorSpecific(void *packet)
{
	struct AV200_MME_VendorSpecific *mme_vs;

	mme_vs = (struct AV200_MME_VendorSpecific*)packet;
	if (!LocalBridgeAlreadyFoundAV200())
	{
		if (!memcmp(p_avTable.localBridge.MAC, mme_vs->header.SourceAddress, MACSIZE))
		{
			printf("\n\tWarning: found another local device. Exiting\n");
			exit(1);
		} else
		{
#ifdef DLANCONFIG_DEBUG
			printf("\n\t\tPrinting MAC\n");
#endif
			AV200_isoldversion = IfOldVersion(mme_vs->data);
			memcpy(p_avTable.localBridge.MAC, mme_vs->header.SourceAddress, MACSIZE);
			p_avTable.localBridge.isHere = 1;
		}
	}
	
}


/*
 *	Gets info about local AV200 lan stored in packet
 */
void GetAV200NetworkInfo(void *packet)
{
	struct AV200_MME_NetworkInfo *mme_nw;
	int stationNumber;
	int offset = 39;
	int i;
	int stationInfoSize;
	struct AV200_MME_StationInfo *mme_si;

	mme_nw = (struct AV200_MME_NetworkInfo*)packet;
	stationNumber = mme_nw->numStations;
	mme_si = (struct AV200_MME_StationInfo*)(packet + offset);
	stationInfoSize = sizeof(struct AV200_MME_StationInfo);
	for ( i = 0; i < stationNumber; i++)
	{		
		GetUpdateDeviceTableAV200(mme_si->address, mme_si->TEI, mme_si->firstBridgedAddress, mme_si->avgTxPhyRateMbps, mme_si->avgRxPhyRateMbps);
		mme_si++;
	}
}

/*
 *	Handle SetKeyCnf response to SetKeyRequest packet
 */
void GetAV200SetKeyCnf(void *packet)
{
	AV200_MME_SetKeyCnf_t *mme_skc;

	mme_skc = (AV200_MME_SetKeyCnf_t*)packet;
	switch (mme_skc->uStatus)
	{
		case (eSUCCESS):
		{
			av200_setlocalnek_succeed = 1;
			break;
		}
		case (eFAIL):
		{
			printf("\n\t\tKeySet fail!\t");
			break;
		}
		case (eINVALID_PEKS):
		{
			printf("\n\t\tKeySet error: invalid PEKS!\t");
			break;
		}
		case (eINVALID_PIB):
		{
			printf("\n\t\tKeySet error: invalid PIB!\t");
			break;
		}
		case (eINVALID_PEKS_PAYLOAD):
		{
			printf("\n\t\tKeySet error: invalid PEKS payload!\t");
			break;
		}
		case (eREMOTE_FAIL):
		{
			printf("\n\t\tKeySet error: remote fail!\t");
			break;
		}
		case (eINVALID_REMOTE_ANSWER):
		{
			printf("\n\t\tKeySet error: invalid remote answer!\t");
			break;
		}
		case (eREMOTE_DECRYPTION_FAILED):
		{
			printf("\n\t\tKeySet error: remote decryption failed!\t");
			break;
		}
	}
}

/*
 *	Gets info about Viper stored in packet
 */
void GetPrivateVendorSpecific(void *packet)
{
	unsigned char chMME;
	private_mme_rsp_t *p;
	p = (private_mme_rsp_t*)packet;
	chMME = p->MID & 0x7f;
	switch(chMME)
	{
		case DEVICE_DESCRIPTION_RESPONSE:
		{
			if(p->vendor.flags & 0x01) //Remote Viper found
			{ 
				GetUpdateDeviceTable(p->SA, ISVIPER);
#ifdef DLANCONFIG_DEBUG
				printf("\n\tPacket Device Description Response\n");
#endif
			}else // Local Viper Found
			{
				if(LocalBridgeAlreadyFound())
				{
					if(memcmp(p_devTable.locBridge.MAC, p->SA, MACSIZE))
					{
						printf("\n\t\tWarning: found another local device. Exiting\n");
						exit(1);
					}
				}else
				{
					memcpy(p_devTable.locBridge.MAC, p->SA, MACSIZE);
					p_devTable.locBridge.devType = ISVIPER;
				}
			}
			break;
		}
		case QUALITY_RESPONSE:
		{
			int i;
			int count;
			int num_entry;
			
			num_entry = ((unsigned int)(p->MMELength - 3))/sizeof(txqualityinfo_t);
			if (!IsSameMAC(p_devTable.locBridge.MAC, p->SA))
				break;
			
			for (i = 0; i < p_devTable.devCounter; i++)
			{
				for (count = 0; count < num_entry; count++)
				{
					if (IsAlmostSameMAC(p_devTable.list[i].MAC, p->txData[count].Address))
					{
						if (p_devTable.list[i].devType == ISVIPER)
							p_devTable.list[i].rate = Viper_DisplayRate(htons(p->txData[count].BytePer40));
						else
							p_devTable.list[i].rate = HP1_DisplayRate(htons(p->txData[count].BytePer40));
						if (IsSameMAC(p_devTable.list[i].MAC, p->txData[count].Address))
							p_devTable.list[i].toggled = 0;
						else
							p_devTable.list[i].toggled = 1;
						break;
					}
				}
			}
			break;
		}
		default:
			break;
	}
}

/*
 *	Gets info about HP1 stored in packet
 */
void GetLocalParExtResp(void* packet)
{
	struct mme_locParExtResp* pHdr;
	pHdr = (struct mme_locParExtResp*) packet;  
	if(LocalBridgeAlreadyFound()){
		if(memcmp(p_devTable.locBridge.MAC,pHdr->SA,MACSIZE))
		{
			printf("\n[WARNING]: Found another Local dLAN device.\n");
			printf("             This software cannot manage more than one local device!\n");
			printf("             sorry, exiting!\n");
			exit(1);
		}
		else
		if(hp1device_localnek_setting==1){
			memcpy(p_devTable.locBridge.NEK,pHdr->NEK,DECKSIZE);    
			hp1device_localnek_setting=0;
		}
	}
	else{
		memcpy(p_devTable.locBridge.MAC,pHdr->SA,MACSIZE);
		memcpy(p_devTable.locBridge.NEK,pHdr->NEK,DECKSIZE);
		memcpy(p_devTable.locBridge.DEK,pHdr->DEK,DECKSIZE);
		p_devTable.locBridge.devType=ISOLDDEVICE;
	}
	
}


/*
 *	Sends a packet to iname interface for Device Description Response
 */
int SendViperPacketMME_DDR(char *iname)
{
	unsigned char mac_destination_address[MACSIZE];
	private_mme_t *pHdr;
	int returnValue = 0;
#ifdef DLANCONFIG_DEBUG
	printf("\n\tSendViperPacketMME_DDR\n");
#endif
	pHdr = (private_mme_t*) malloc(MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	memset(pHdr, 0x00, MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	memset(mac_destination_address, 0xff, MACSIZE);
	
	pHdr->numEntries = 0x01;
	pHdr->MMEHeader = 0x02;
	pHdr->MMELength = 0x04;
	pHdr->MMEOUI[0] = 0x00;
	pHdr->MMEOUI[1] = 0x04;
	pHdr->MMEOUI[2] = 0x87;
	pHdr->MID = DEVICE_DESCRIPTION_RESPONSE;
	returnValue = SendPacket(iname, mac_destination_address, pHdr, MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	free(pHdr);
	return returnValue;
}


/*
 * 	Sends a packet to get info on HP1
 */
int SendHP1ParamPacket(char *iname)
{  
	unsigned char mac_dest_address[MACSIZE];  
	struct mme_hdr* pHdr;
	int returnValue=0;
#ifdef DLANCONFIG_DEBUG	
	printf("\n\tSendHP1ParamPacket\n");
#endif
	pHdr = (struct mme_hdr*) malloc(MME_NET_STAT_FRAME_LEN-MME_HDR_LEN);
	memset(mac_dest_address,0xff,MACSIZE); 
	memset(pHdr,0x00,MME_NET_STAT_FRAME_LEN-MME_HDR_LEN);
	
	pHdr->MCTRL = 0x01 ;                 
	pHdr->MEHDR = CODE_SetLocPar ;      
	pHdr->MELEN = MME_LocalParExtSize;  
	
	returnValue = SendPacket(iname, mac_dest_address,pHdr,MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	free(pHdr);
	return returnValue;	  
}

/*
 *	Send a packet to iname interface for a stats request
 */
int SendStatRequest(char *iname)
{
	unsigned char mac_destination_address[MACSIZE];
	unsigned char outFrame[MME_MIN_FRAME_LEN - MME_HDR_LEN];
	int returnValue = 0;

	memset(outFrame, 0x00, MME_MIN_FRAME_LEN - MME_HDR_LEN);
	memset(mac_destination_address, 0xff, MACSIZE);
	
	outFrame[0] = 0x01;
	outFrame[1] = CODE_ReqStats;
	outFrame[2] = MME_ReqStatsSize;
	outFrame[3] = 0x00;

	returnValue = SendPacket( iname, mac_destination_address, outFrame, MME_MIN_FRAME_LEN - MME_HDR_LEN);
	return returnValue;
}

/*
 *	Sends a packet using the MME of the devoloAV200
 */
int SendPacketAV200(char *iname,unsigned char addr[],void *buffer,size_t len)
{
	int i = 0;
	int returnValue = 0;
	struct sockaddr_ll destAddr;
	struct sockaddr_ll sourceAddr;
	int sock = 0;
	struct ifreq ifr;

	memset(&sourceAddr, 0, sizeof(sourceAddr));
	memset(&destAddr, 0, sizeof(destAddr));
	strcpy(ifr.ifr_name,iname);

	sock = socket(PF_PACKET, SOCK_DGRAM, htons(SOCK_PROTO));
	if ( sock < 0 )
	{
		printf("[ERROR]: SendPacket: socket unable to send packet \n");
		return -1;
	}

	if (ioctl(sock,SIOCGIFINDEX,&ifr))
	{
		printf("[ERROR]: SendPacket: error ioctl SIOCGIFINDEX errno=%d\n",errno);
		close(sock);
	return -1;
	}

	destAddr.sll_family = AF_PACKET;
	destAddr.sll_protocol =htons(ETHERTYPE_AV200);
	destAddr.sll_ifindex= ifr.ifr_ifindex;
	destAddr.sll_halen = 6;

	for(i=0;i<MACSIZE;i++)
		destAddr.sll_addr[i]=addr[i];

	sourceAddr.sll_family = PF_PACKET;
	sourceAddr.sll_protocol=htons(SOCK_PROTO);
	sourceAddr.sll_ifindex = ifr.ifr_ifindex;
	returnValue = bind(sock,(struct sockaddr *) &sourceAddr, sizeof(sourceAddr));
	if ( returnValue==-1)
	{
		perror("SendPacket: bind error\n");
	}else
		returnValue = sendto(sock, buffer, len, 0,(struct sockaddr *) &destAddr,sizeof(destAddr));

	if ( returnValue==-1)
		perror("SendPacket: sendto error\n");
#ifdef DLANCONFIG_DEBUG
	printf("\n\tSent packet to %2x:%2x:%2x:%2x:%2x:%2x\n", addr[0], addr[1], addr[2], addr[3], addr[4], addr[5]);
#endif
	close(sock);
	return(returnValue);
}

/*
 *	Update the device table with all the info of the new AV200
 */
void GetUpdateDeviceTableAV200(unsigned char *mac, unsigned char tei, unsigned char *firstBridgedMac, unsigned char txRate, unsigned char rxRate)
{
	int i = 0;
	// Control if the mac is already present
	if (IsSameMAC(mac, p_avTable.localBridge.MAC))
		return;
	// Scan all the entry of the list to seek if the mac is already present
	for (i = 0; i < p_avTable.devCounter; i++)
	{
		if (IsSameMAC(p_avTable.list[i].MAC, mac))
		{
			p_avTable.list[i].isHere = 1;
#ifdef DLANCONFIG_DEBUG
			printf("\n\t\tDevice already found\n");
#endif
			return;
		}
	}
#ifdef DLANCONFIG_DEBUG
	printf("added device: %X%X%X%X%X%X \n",mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
#endif
	memcpy(p_avTable.list[p_avTable.devCounter].MAC, mac, MACSIZE);
	p_avTable.list[p_avTable.devCounter].TEI = tei;
	memcpy(p_avTable.list[p_avTable.devCounter].FirstBridgedMAC, firstBridgedMac, MACSIZE);
	p_avTable.list[p_avTable.devCounter].avgTxPhyRateMbps = txRate;
	p_avTable.list[p_avTable.devCounter].avgRxPhyRateMbps = rxRate;
	p_avTable.list[p_avTable.devCounter].isHere = 1;
	p_avTable.devCounter++;

}


/*
 *	Update the device table with the mac of the new device
 */
void GetUpdateDeviceTable(unsigned char *mac, unsigned char deviceType)
{
	int i = 0;
	// Controls if the mac is already present
	if (IsSameMAC(mac, p_devTable.locBridge.MAC))
		return;
	
	for (i = 0; i < p_devTable.devCounter; i++)
	{
		if (IsSameMAC(p_devTable.list[i].MAC, mac))
		{
			p_devTable.list[i].isHere = 1;
			if (deviceType == ISVIPER)
				p_devTable.list[i].devType = deviceType;
			return;
		}
	}
#ifdef DLANCONFIG_DEBUG
	printf("add device: %X%X%X%X%X%X devtype=%d\n",mac[0],mac[1],mac[2],mac[3],mac[4],mac[5],deviceType);
#endif
	memcpy(p_devTable.list[p_devTable.devCounter].MAC, mac, MACSIZE);
	p_devTable.list[p_devTable.devCounter].devType = deviceType;
	p_devTable.list[p_devTable.devCounter].isHere = 1;
	p_devTable.devCounter++;

}

/*
 *	Sends a packet to iname interface for Quality Info Request
 */
int SendViperPacketMME_QIR(char *iname)
{
	unsigned char mac_destination_address[MACSIZE];
	private_mme_t *pHdr;
	int returnValue = 0;
	
	pHdr = (private_mme_t*) malloc(MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	memset(pHdr, 0x00, MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	memset(mac_destination_address, 0xff, MACSIZE);
	
	pHdr->numEntries = 0x01;
	pHdr->MMEHeader = 0x02;
	pHdr->MMELength = 0x05;
	pHdr->MMEOUI[0] = 0x00;
	pHdr->MMEOUI[1] = 0x04;
	pHdr->MMEOUI[2] = 0x87;
	pHdr->MID = QUALITY_RESPONSE;
	pHdr->data[0] = 0x01;
	returnValue = SendPacket(iname, mac_destination_address, pHdr, MME_NET_STAT_FRAME_LEN - MME_HDR_LEN);
	free(pHdr);
	return returnValue;
}

/*
 *	Send a packet to iname interface for a network stats request
 */
int SendNetStatsRequest(char *iname)
{
	unsigned char mac_destination_address[MACSIZE];
	unsigned char outFrame[MME_MIN_FRAME_LEN - MME_HDR_LEN];
	int returnValue = 0;

	memset(outFrame, 0x00, MME_MIN_FRAME_LEN - MME_HDR_LEN);
	memset(mac_destination_address, 0xff, MACSIZE);
	
	outFrame[0] = 0x01;
	outFrame[1] = CODE_ReqNetStats;
	outFrame[2] = 0xbb;
	outFrame[3] = 0x80;

	returnValue = SendPacket( iname, mac_destination_address, outFrame, MME_MIN_FRAME_LEN - MME_HDR_LEN);
	return returnValue;
}

/*
 *	Send packet to set local nek
 */

int SendSetLocalNek(char *iname, unsigned char* newNek){  

	unsigned char mac_dest_addr[MACSIZE];  
	struct mme_double* pHdr; 
	unsigned char* pKey;
	int returnValue = 0;
	
	pHdr=(struct mme_double*) malloc(MME_MIN_FRAME_LEN - MME_HDR_LEN);
	//  memset(mac_dest_addr,0xff,MACSIZE);
	memcpy(mac_dest_addr,p_devTable.locBridge.MAC, MACSIZE);
	memset(pHdr, 0x00, MME_MIN_FRAME_LEN - MME_HDR_LEN) ; 
	
	pHdr->CTRL = 0x02;   
	pHdr->HDR1 = 0x04;   
	pHdr->LEN1 = 0x09;   
	pHdr->EKS = 0x01;   
	
	pKey = MD5HomePlugKey((char*)newNek);
	memcpy(pHdr->NEK, pKey, DECKSIZE);
	if (p_devTable.locBridge.devType != ISVIPER){
	
		#ifdef KEEP_MME_SETCHAR
		/* change characteristics.:  removed to fix bug 1082....*/
		pHdr->HDR2      = 0x1F;
		pHdr->LEN2      = 0x03;
		pHdr->TXCHAR[0] = 0x58;   
		pHdr->TXCHAR[1] = 0x84;
		#ifdef KEEP_ENABLE_BACKPRESSURE
		pHdr->TXCHAR[1] |= 0x02;
		#endif   
		pHdr->TXCHAR[2] = 0x01;     
		returnValue = SendPacket(iname, mac_dest_addr, pHdr, MME_MIN_FRAME_LEN - MME_HDR_LEN); 
		#else
		returnValue = SendPacket(iname, mac_dest_addr, pHdr, MME_MIN_FRAME_LEN - MME_HDR_LEN - 5);
		#endif
	}
	else
	returnValue = SendPacket(iname, mac_dest_addr, pHdr, MME_MIN_FRAME_LEN - MME_HDR_LEN - 5);
	free(pHdr); 
	return returnValue;	  
}

/*
 *	Send packet to set remote nek on hp1 device
 */
int SendSetRemoteHP1Nek(char *iname, unsigned char* newNek, unsigned char* remoteDek)
{ 
	unsigned char mac_destination_address[MACSIZE]; 
	struct mme_double* mme_d; 
	struct mme_triple* mme_tr;
	unsigned char* pKey;
	int returnValue = 0;
	
	mme_d = (struct mme_double*) malloc(MME_MIN_FRAME_LEN);
	mme_tr = (struct mme_triple*) malloc(MME_MIN_FRAME_LEN);
	
	memset(mac_destination_address, 0xff, MACSIZE);
	memset(mme_d, 0x00, MME_MIN_FRAME_LEN) ; 
	
	mme_d->CTRL  = 0x02;   
	mme_d->HDR1  = 0x04;   
	mme_d->LEN1  = 0x09;   
	mme_d->EKS   = 0x00;   
	
	pKey = MD5HomePlugKey((char*)remoteDek);
	memcpy(mme_d->NEK, pKey, DECKSIZE);  
	
	#ifdef KEEP_MME_SETCHAR
	/* change characteristics.:  removed to fix bug 1082....*/
	mme_d->HDR2      = 0x1F;
	mme_d->LEN2      = 0x03;
	mme_d->TXCHAR[0] = 0x58;   
	mme_d->TXCHAR[1] = 0x84;
	#ifdef KEEP_ENABLE_BACKPRESSURE
	mme_d->TXCHAR[1] |= 0x02;
	#endif   
	mme_d->TXCHAR[2] = 0x00;
	returnValue = SendPacket(iname, mac_destination_address, mme_d, MME_MIN_FRAME_LEN - MME_HDR_LEN); 
	#else
	returnValue = SendPacket(iname, mac_destination_address, mme_d, MME_MIN_FRAME_LEN - MME_HDR_LEN - 5); 
	#endif
	
	
	if(returnValue <= 0){
		printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
		free(mme_d);
		free(mme_tr);
		return(returnValue);
	}
	
	memset(mac_destination_address, 0xff, MACSIZE); 
	memset(mme_d, 0x00, MME_MIN_FRAME_LEN - MME_HDR_LEN);
	
	// added check response  mme_d->CTRL  = 0x01;  
	mme_d->CTRL  = 0x02;  
	mme_d->HDR1  = 0x04;  
	mme_d->LEN1  = 0x09;  
	mme_d->EKS   = 0x01;  
	
	pKey = MD5HomePlugKey((char*)newNek); 
	memcpy(mme_d->NEK, pKey, DECKSIZE);  
	
	// added check response
	mme_d->HDR2   =   CODE_ReqStats;
	mme_d->LEN2   =   MME_ReqStatsSize; 
	mme_d->TXCHAR[0] = 0x00;   
	
	returnValue = SendPacket(iname, mac_destination_address, mme_d, MME_MIN_FRAME_LEN - MME_HDR_LEN); 
	if (returnValue <= 0)
	{
		printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
		free(mme_d);
		free(mme_tr);
		return(returnValue);
	}
	
	memset(mac_destination_address, 0xff, MACSIZE); 
	memset(mme_tr, 0x00, MME_MIN_FRAME_LEN) ; 
	
	mme_tr->CTRL  = 0x03;   
	mme_tr->HDR1  = 0x04;   
	mme_tr->LEN1  = 0x09;   
	mme_tr->EKS1  = 0x00;   
	memcpy(mme_tr->NEK1, p_devTable.locBridge.DEK, DECKSIZE);
	
	mme_tr->HDR2  = 0x04; 
	mme_tr->LEN2  = 0x09; 
	mme_tr->EKS2  = 0x01;
	memcpy(mme_tr->NEK2, p_devTable.locBridge.NEK, DECKSIZE);
	
	
	#ifdef KEEP_MME_SETCHAR
	/* change characteristics.:  removed to fix bug 1082....*/
	mme_tr->HDR3  = 0x1F;
	mme_tr->LEN3  = 0x03;
	mme_tr->TXCHAR[0] = 0x58;  
	mme_tr->TXCHAR[1] = 0x84;
	#ifdef KEEP_ENABLE_BACKPRESSURE
	mme_d->TXCHAR[1] |= 0x02;
	#endif  
	mme_tr->TXCHAR[2] = 0x01;
	returnValue = SendPacket(iname, mac_destination_address, mme_tr, MME_MIN_FRAME_LEN - MME_HDR_LEN); 
	#else
	returnValue = SendPacket(iname, mac_destination_address, mme_tr, MME_MIN_FRAME_LEN - MME_HDR_LEN - 5); 
	#endif
	
	if (returnValue <= 0)
	{
		printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
		free(mme_d);
		free(mme_tr);
		return(returnValue);
	}
	
	memset(mac_destination_address, 0xff, MACSIZE); 
	memset(mme_d, 0x00, MME_MIN_FRAME_LEN - MME_HDR_LEN);
	mme_d->CTRL  = 0x02;  
	mme_d->HDR1  = 0x04;  
	mme_d->LEN1  = 0x09;  
	mme_d->EKS   = 0x01; 
	memcpy(mme_d->NEK, p_devTable.locBridge.NEK, DECKSIZE);
	
	#ifdef KEEP_MME_SETCHAR
	/* change characteristics.:  removed to fix bug 1082....*/
	mme_d->HDR2      = 0x1F;
	mme_d->LEN2      = 0x03;
	mme_d->TXCHAR[0] = 0x58;   
	mme_d->TXCHAR[1] = 0x84;
	#ifdef KEEP_ENABLE_BACKPRESSURE
	mme_d->TXCHAR[1] |= 0x02;
	#endif
	mme_d->TXCHAR[2] = 0x01;
	
	returnValue = SendPacket(iname, mac_destination_address, mme_d, MME_MIN_FRAME_LEN - MME_HDR_LEN); 
	#else
	returnValue = SendPacket(iname, mac_destination_address, mme_d, MME_MIN_FRAME_LEN - MME_HDR_LEN - 5); 
	#endif
	if (returnValue <= 0)
	{
		printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
		free(mme_d);
		free(mme_tr);
		return(returnValue);
	}
	free(mme_d);
	free(mme_tr);
	return returnValue;
}

/*
 *	Set remote viper nek
 */
int SendSetRemoteViperNek(char *iname, unsigned char* newNek, unsigned char* remoteDek)
{
	unsigned char mac_destination_address[MACSIZE];  
	MAC_MME_MSG_t* pHdr1;
	mme_setremoteviper_t* pHdr2;
	unsigned char* pnKey;
	unsigned char* prKey;
	int returnValue = 0;
	int dwLen = 0;
	int framesize = MME_MIN_FRAME_LEN - MME_HDR_LEN;

	if (sizeof(MAC_MME_MSG_t) > framesize)
		framesize = sizeof(MAC_MME_MSG_t);
	pHdr1 = (MAC_MME_MSG_t*) malloc(framesize);
	memset(mac_destination_address, 0xff, MACSIZE);
	memset(pHdr1, 0x00, framesize); 
	if(remoteDek)
	{
    		pHdr1->mu8Ctrl = 1;
    		pHdr1->mu8Header = 0x14;
    		pHdr1->mu8Len = 10;
    		pHdr1->mu8MsgID = 1;
    		pHdr1->mu8EKS = 0;
    		prKey = MD5HomePlugKey((char*)remoteDek);
    		memcpy(pHdr1->mu8DEK, prKey ,DECKSIZE);
		dwLen = 13;
	}
	pHdr2 = (mme_setremoteviper_t*)(pHdr1->muData);
	pHdr2->Ethertype = htons(ETHERTYPE_INTELLON);
	pHdr2->CTRL = 2;
	pHdr2->HDR = 0x04;
	pHdr2->LEN = 9;
	pHdr2->EKS = 1;
	pnKey = MD5HomePlugKey((char*)newNek);
	memcpy(pHdr2->NEK, pnKey, DECKSIZE);
  
	pHdr2->HDR2   =   CODE_ReqStats;
	pHdr2->LEN2   =   MME_ReqStatsSize; 
	pHdr2->DUM2   =   0;

	returnValue = SendPacket(iname, mac_destination_address, pHdr1, framesize);
	free(pHdr1); 
	return returnValue; 
}



int API_NetwPoll(char *iname)
{
	int count;
	int returnValue = 0;
	
	for (count = 0; count < POLL_RETRY; count++)
	{
		returnValue += SendStatRequest(iname);
		sleep(INTER_FRAME_DELAY);
		returnValue += SendNetStatsRequest(iname);
	}
	return returnValue;
}
/*
 *	Update physical data rates of the devices
 */
void GetUpdatePhyDataRates(void *packet)
{
	struct API_ReqNetStatsResp *p;
	int i, count;
	double rate = 0.0;
	
	p = (struct API_ReqNetStatsResp*)packet;
	
	for (i = 0; i < p_devTable.devCounter; i++)
	{
		for (count = 0; count < NWSIZE; count++)
		{
			if (IsAlmostSameMAC(p_devTable.list[i].MAC, p->NW[count].DA))
			{
				rate = ((p->NW[count].BYTES40) / 42.0);
				rate = ( M_RATE * rate) + B_RATE;
				p_devTable.list[i].rate = rate;
			
				if (IsSameMAC(p_devTable.list[i].MAC, p->NW[count].DA))
					p_devTable.list[i].toggled = 0;
				else
					p_devTable.list[i].toggled = 1;
				break;
			}
		}
	}
}

/*
 *	Search for devices on local dLAN
 */
int LookDevices(char* iname, int pipe)
{
	int returnValue;
	printf("\n\tSearching for devices on local dLAN.");
	returnValue = SendStatRequest(iname);
	if (returnValue < 0)
	{
		printf("\n\tUnable to send packets!\n");
		return 1;
	}
	sleep(INTER_FRAME_DELAY);
	printf(".");
	fflush(stdout);
	returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
	if (returnValue < 0)
	{
		printf("\n\tUnable to receive packets!\n");
		return 1;
      	}
      	printf(".");
	fflush(stdout);
	usleep(INTER_FRAME_DELAY_USEC);

	returnValue = SendViperPacketMME_DDR(iname);
	if (returnValue < 0)
	{
		printf("\n\tUnable to send packets!\n");
		return 1;
	}
	usleep(INTER_FRAME_DELAY_USEC);
	printf(".");
	fflush(stdout);
	returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
	if (returnValue < 0)
	{
		printf("\n\tUnable to receive packets!\n");
		return 1;
	}
	if (p_devTable.locBridge.devType == ISOLDDEVICE)
	{
		returnValue = SendNetStatsRequest(iname);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
	}else if (p_devTable.locBridge.devType == ISVIPER)
	{
		returnValue = SendViperPacketMME_QIR(iname);
		if ( returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
	} else
	{
		printf("\n\tLocal Bridge is not recognized!");
		return 1;
	}
	printf(".");
	fflush(stdout);
	sleep(INTER_FRAME_DELAY);
	returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
	if (returnValue < 0)
	{
		printf("\n\tUnable to receive packets!\n");
		return 1;
	}
	printf(".\n");
	fflush(stdout);
	
	return 0;
}

/*
 *	Get the MAC of the local machine
 */
int GetLocalMAC(char *iname, int pipe)
{
	int returnValue, retry, i;
	returnValue = 0;
	printf("\n\tSearching for local device.");
	
	for ( retry = MAXRETRY; retry > 0; retry--)
	{
		returnValue = SendAV200GetVersionRequest(iname);
		if (returnValue < 0)
		{
			printf("\nError sending packets!\n");
			return 1;
		}
		printf(".");
		fflush(stdout);
		sleep(INTER_FRAME_DELAY);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (returnValue < 0)
		{
			printf("\nError receiving packets!\n");
			return 1;
		}
	}
	
	if (LocalBridgeAlreadyFoundAV200())
	{
		printf("\n\n\tLocal AV200 bridge found!\n");
		return 0;
	}
	// Search for local devices
	for( retry = MAXRETRY; retry; retry--)
	{
		// Search for Viper
		returnValue = SendViperPacketMME_DDR(iname);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
		usleep(INTER_FRAME_DELAY_USEC);
		printf(".");
		fflush(stdout);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE,WAIT_AFTER_NOACTIVITY);
		if(returnValue < 0)
		{
			printf("\n\tUnable to receive packets!\n");
			return 1;
    		}
  		printf(".");
		fflush(stdout);
		// Search for HP1
		for ( i = 0; i < LOCAL_POLL_RETRY; i++)
		{	
			returnValue = SendHP1ParamPacket(iname);
			if (returnValue < 0)
			{
				printf("\n\tUnable to send packets!\n");
				return 1;
			}
			usleep(INTER_FRAME_DELAY_USEC);
			printf(".");
			fflush(stdout);
			returnValue = ListenForPackets(pipe,MAX_PACKET_TO_PARSE,WAIT_AFTER_NOACTIVITY);
			if (returnValue < 0)
			{
				printf("\n\tUnable to receive packets!\n");
				return 1;
			}
			printf(".");
			fflush(stdout);
		}
		if (LocalBridgeAlreadyFound())
		{
			printf("\n\tLocal bridge found!\n");
			return 0;
		}
	}

	return 1;
}

/*
 *	Change the local NEK of the device
 */
int SetLocalNEK(char *iname, int pipe, unsigned char *newNek)
{
	int returnValue = 0;
	viper_setlocalnek_succeed = 0;
	hp1device_localnek_setting = 1;	
	
	printf("\n\tWaiting for response.");
	returnValue = SendSetLocalNek(iname, newNek);
	if (returnValue < 0)
	{
		printf("\n\tUnable to send packets!\n");
		return 1;
	}
	usleep(INTER_FRAME_DELAY_USEC);
	printf(".");
	fflush(stdout);
	if (p_devTable.locBridge.devType == ISVIPER)
	{
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE,WAIT_AFTER_NOACTIVITY);
		printf(".");
		fflush(stdout);
		if(returnValue < 0)
		{
			returnValue = SendSetLocalNek(iname, newNek);
			if (returnValue < 0)
			{
				printf("\n\tUnable to send packets!\n");
				return 1;
			}
			usleep(INTER_FRAME_DELAY_USEC);
			returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE,WAIT_AFTER_NOACTIVITY);
			if (returnValue < 0)
			{
				printf("\n\tUnable to receive packets!\n");
				return 1;
			}
      		}
		if (viper_setlocalnek_succeed == 0)
		{
			printf("\n\tUnable to change local password!\n");
			return 1;
		}
      		
	} else {
		returnValue = SendHP1ParamPacket(iname);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
		usleep(INTER_FRAME_DELAY_USEC);
		printf(".");
		fflush(stdout);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (hp1device_localnek_setting == 0)
		{
			if(memcmp(p_devTable.locBridge.NEK, MD5HomePlugKey((char*)newNek), DECKSIZE))
			{
				printf("\n\tUnable to change local password!\n");
				return 1;
			}
		}
	}
	
	return 0;
}

/*
 *	Change the remote NEK of another device
 */
int SetRemoteNEK(char *iname, int pipe, unsigned char *newNek, unsigned char *deviceDek)
{
	int returnValue = 0;

	printf("\n\tWaiting for response.");
	if (p_devTable.locBridge.devType == ISVIPER)
	{
		viper_setlocalnek_succeed = 0;
		returnValue = SendSetRemoteViperNek(iname, newNek, deviceDek);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!");
			return 1;
		}
		printf(".");
		fflush(stdout);
		usleep(INTER_FRAME_DELAY_USEC);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (returnValue < 0)
		{
			returnValue = SendSetRemoteViperNek(iname, newNek, deviceDek);
			if (returnValue < 0)
			{
				printf("\n\tUnable to send packets!");
				return 1;
			}	
			printf(".");
			fflush(stdout);
			usleep(INTER_FRAME_DELAY_USEC);
			returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
			if (returnValue < 0)
			{
				printf("\n\tUnable to receive packets!");
				return 1;
			}	
		}
		if (viper_setlocalnek_succeed == 0)
		{
			printf("\n\tUnable to change remote password!\n");
			printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
			return 1;
		}		
	
	}else {
		viper_setlocalnek_succeed = 0;
		returnValue = SendSetRemoteHP1Nek(iname, newNek, deviceDek);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packet!");
			return 1;
		}
		printf(".");
		fflush(stdout);
		usleep(INTER_FRAME_DELAY_USEC);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (returnValue < 0)
		{
			returnValue = SendSetRemoteHP1Nek(iname, newNek, deviceDek);
			if (returnValue < 0)
			{
				printf("\n\tUnable to send packets!");
				return 1;
			}	
			printf(".");
			fflush(stdout);
			returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
			if (returnValue < 0)
			{
				printf("\n\tUnable to receive packets!");
				return 1;
			}
		}
		if (viper_setlocalnek_succeed == 0)
		{
			printf("\n\tUnable to change remote password!\n");
			printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
			return 1;
		}
	}
	return 0;
}

/*
 *	Sends NetworkInfo Requests and listen for NetworkInfo Responses
 */
int LookDevicesAV200(char *iname, int pipe)
{
	int returnValue = 0;
	int i;
	printf("\n\tSearching for devices on local dLAN.");
	for ( i = 0; i < 3; i++)
	{
		returnValue = SendAV200NetworkInfoRequest(iname);
		usleep(INTER_FRAME_DELAY_USEC);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
		usleep(INTER_FRAME_DELAY_USEC);
		sleep(INTER_FRAME_DELAY);
		printf(".");
		fflush(stdout);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (returnValue < 0)
		{
			printf("\n\tUnable to receive packets!\n");
			return 1;
		}
		printf(".");
		fflush(stdout);
		usleep(INTER_FRAME_DELAY_USEC);
	}
	printf("\n");
	return returnValue;

}

/*
 *	Send NetworkInfo Request packet to local AV200
 */
int SendAV200NetworkInfoRequest(char *iname)
{
	unsigned char mac_dest_address[MACSIZE];  
	struct AV200_MME *pHdr;
	int returnValue = 0;
	int length = MME_MIN_FRAME_LEN - MME_HDR_LEN - 4;

	pHdr = (struct AV200_MME*)malloc(length);
	memcpy(mac_dest_address, p_avTable.localBridge.MAC, MACSIZE);
	memset(pHdr, 0, length);
	pHdr->MMVersion = MME_VERSION_AV200;
	pHdr->MMEType[0] = 0x38;
	pHdr->MMEType[1] = 0xa0;
	memcpy(pHdr->IntellonOUI, cIntellonOUI, 3);
	
	returnValue = SendPacketAV200( iname, mac_dest_address, pHdr, length);
	free(pHdr);
	
	return returnValue;

}


/*
 *	Send a GetVersion Request MME packet to local AV200
 */
int SendAV200GetVersionRequest(char *iname)
{
	
	unsigned char mac_dest_address[MACSIZE];  
	struct AV200_MME *pHdr;
	int length = MME_MIN_FRAME_LEN - MME_HDR_LEN - 4;
	int returnValue = 0;
	
	pHdr = (struct AV200_MME*)malloc(length);
	memcpy(mac_dest_address, default_destination_MAC, MACSIZE);
	memset(pHdr, 0, length);
	pHdr->MMVersion = MME_VERSION_AV200;
	pHdr->MMEType[0] = 0x00;
	pHdr->MMEType[1] = 0xa0;
	memcpy(pHdr->IntellonOUI, cIntellonOUI, 3);	

	returnValue = SendPacketAV200( iname, mac_dest_address, pHdr, length);
	free(pHdr);
	return returnValue;
	
}



/*
 *	Handle the send and the listening of the local SetKey packet
 */
int SetLocalNEKAV200(char *iname, int pipe, unsigned char *newNek)
{
	int returnValue = 0;
	int i;

	av200_setlocalnek_succeed = 0;
	printf("\n\tWaiting for response.");
	for ( i = 0; i < 2; i++)
	{
		returnValue = SendLocalAV200SetKeyRequest(iname, newNek);
		usleep(INTER_FRAME_DELAY_USEC);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
		usleep(INTER_FRAME_DELAY_USEC);
		sleep(INTER_FRAME_DELAY*3);
		printf(".");
		fflush(stdout);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (returnValue < 0)
		{
			printf("\n\tUnable to receive packets!\n");
			return 1;
		}
		printf(".");
		fflush(stdout);
	}
	if (av200_setlocalnek_succeed == 0)
	{
		printf("\n\tUnable to change local password!\n");
		returnValue = 0;
	}
	printf("\n");
#ifdef DLANCONFIG_DEBUG
	printf("\n\t\t SetLocalNEK returns %i\n", returnValue);
#endif
	return returnValue;	
}

/*
 *	Handle the send and listening of the Remote SetKey packet 
 */
int SetRemoteNEKAV200(char *iname, int pipe, unsigned char *newNek, unsigned char *deviceDek)
{
	int returnValue = 0;
	int i;

	printf("\n\tWaiting for response.");
	for ( i = 0; i < 2; i++)
	{
		returnValue = SendRemoteAV200SetKeyRequest(iname, newNek, deviceDek);
		usleep(INTER_FRAME_DELAY_USEC);
		if (returnValue < 0)
		{
			printf("\n\tUnable to send packets!\n");
			return 1;
		}
		usleep(INTER_FRAME_DELAY_USEC);
		printf(".");
		fflush(stdout);
		returnValue = ListenForPackets(pipe, MAX_PACKET_TO_PARSE, WAIT_AFTER_NOACTIVITY);
		if (returnValue < 0)
		{
			printf("\n\tUnable to receive packets!\n");
			return 1;
		}
		printf(".");
		fflush(stdout);
		usleep(INTER_FRAME_DELAY_USEC);
	}
	if (av200_setlocalnek_succeed == 0)
	{
		printf("\n\tUnable to change remote password!\n");
		printf("\tPlease check that the dLAN adapter's security ID is correct or/and that it is NOT in standby mode.\n");
		returnValue = 0;
	}
	printf("\n");
#ifdef DLANCONFIG_DEBUG
	printf("\n\t\t SetRemoteNEK returns %i\n", returnValue);
#endif
	return returnValue;
}

/*
 *	Send a SetKey Request MME packet to local AV200
 */
int SendLocalAV200SetKeyRequest(char *iname, unsigned char *newNek)
{
	unsigned char mac_dest_address[MACSIZE];  
	AV200_MME_SetKey_t* pHdr;
	int returnValue = 0;
	unsigned char* key;
	
	pHdr = (AV200_MME_SetKey_t*)malloc(sizeof(AV200_MME_SetKey_t));
	memset(pHdr, 0x00, sizeof(AV200_MME_SetKey_t));
	key = (unsigned char*)malloc(16);
	NetworkKeyFromPassword(newNek, key);		
	memcpy(mac_dest_address, p_avTable.localBridge.MAC, MACSIZE);
	pHdr->header.MMVersion = MME_VERSION_AV200;
	pHdr->header.MMEType[0] = 0x50;
	pHdr->header.MMEType[1] = 0xa0;
	memcpy(pHdr->header.IntellonOUI, cIntellonOUI, 3);
	if(AV200_isoldversion)
	  pHdr->body.uPEKS = 0x03;		
	else
	  pHdr->body.uPEKS = 0x01;
	// pHdr->body.uPEKS = 0x03;		
	memcpy(pHdr->body.cKEY, key, 16);
	pHdr->body.uPEKS_EncPayload = 0x0f;

	returnValue = SendPacketAV200( iname, mac_dest_address, pHdr, MME_MIN_FRAME_LEN - MME_HDR_LEN - 4);
	free(pHdr);
	free(key);
#ifdef DLANCONFIG_DEBUG
	printf("\n\t\t SendLocalSetKey returns %i\n", returnValue);
#endif
	return returnValue;

}

/*
 *	Send a remote SetKey Request MME packet to local AV200
 */
int SendRemoteAV200SetKeyRequest(char *iname, unsigned char *newNek, unsigned char *deviceDek)
{
	
	unsigned char mac_dest_address[MACSIZE];  
	AV200_MME_SetKey_t* pHdr;
	int returnValue = 0;
	unsigned char *key;// = "";
	unsigned char *dak;// = "";
	
	pHdr = (AV200_MME_SetKey_t*)malloc(sizeof(AV200_MME_SetKey_t));
	key = (unsigned char *)malloc(16);
	dak = (unsigned char *)malloc(16);
	memcpy(mac_dest_address,  p_avTable.localBridge.MAC, MACSIZE);
	memset(pHdr, 0, sizeof(struct AV200_MME_SetKey));
	NetworkKeyFromPassword(newNek, key);
	pHdr->header.MMVersion = MME_VERSION_AV200;
	pHdr->header.MMEType[0] = 0x50;
	pHdr->header.MMEType[1] = 0xa0;
	memcpy(pHdr->header.IntellonOUI, cIntellonOUI, 3);
	if(AV200_isoldversion)
	  pHdr->body.uPEKS = 0x03;		
	else
	  pHdr->body.uPEKS = 0x01;
	//pHdr->body.uPEKS = 3;		
	memcpy(pHdr->body.cKEY, key, 16);
	pHdr->body.uPEKS_EncPayload = 0x00;
	memset(pHdr->body.cRemoteMacAddr, 0xff, MACSIZE);
	DeviceAccessKeyFromSecurityID(deviceDek, dak);
	memcpy(pHdr->body.cKEY_EncPayload, dak, 16);

	returnValue = SendPacketAV200( iname, mac_dest_address, pHdr, MME_MIN_FRAME_LEN - MME_HDR_LEN - 4);
	free(pHdr);
	free(key);
#ifdef DLANCONFIG_DEBUG
	printf("\n\t\t SetRemoteSetKey returns %i\n", returnValue);
#endif
	return returnValue;

}
/*
 *	Close all the processes and the pipe
 */
void CloseAll(int fileDesc, pid_t pid)
{
	close(fileDesc);
	kill(pid, 15);
	wait(NULL);
}

/*
 *	Creates the pipe, fork the process and controls all the actions
 */
pid_t ForkProcesses(char action,char *data, unsigned char* newNek, unsigned char* remoteDek)
{
	pid_t pid;
	int returnValue = 0;
	int newpipe[2];
	char* argv[3];

	argv[0] = SNIFFING_CHILD_PROCESS;
	argv[1] = data;
	argv[2] = NULL;
	errno = 0;
	// Creates the pipe
	if (pipe(newpipe) != 0)
	{
		printf("\n Error-> ForkProcesses: error creating the pipe\n");
		strerror(errno);
		return -1;
	}
	// Fork
	if ((pid = fork()) == 0)
	{
		dup2(newpipe[1], STDOUT_FILENO);
		close(newpipe[0]);
		close(newpipe[1]);
		errno = 0;
		execvp(SON_PROCESS_NAME, (char**)argv);
		fprintf(stderr,"cf: %s return\n",SNIFFING_CHILD_PROCESS);
		strerror(errno);
		fflush(stdout);
		// Returns a different error
		return -2;
	}
	if (action == GETLOCALMAC)
	{
		returnValue = GetLocalMAC(data, newpipe[0]);
		if (returnValue != 0)
		{
			printf("\n\n\tNo local device found, exiting!\n\n");
			CloseAll(newpipe[0], pid);
			exit(1);
		}
		CloseAll(newpipe[0], pid);
	}
	if (p_avTable.localBridge.isHere == 1)
	{
		switch (action)
		{
			case POLL:
				returnValue = LookDevicesAV200(data, newpipe[0]);		
				CloseAll(newpipe[0], pid);
				return returnValue;
				break;
			case LOCAL:
				returnValue = SetLocalNEKAV200(data, newpipe[0], newNek);
				CloseAll(newpipe[0], pid);
				return returnValue;
				break;
			case REMOTE:
				returnValue = SetRemoteNEKAV200(data, newpipe[0], newNek, remoteDek);
				CloseAll(newpipe[0], pid);
				return returnValue;
				break;
		}
	}else
	{
		switch (action)
		{
			case POLL:	
				returnValue = LookDevices(data, newpipe[0]);
				CloseAll(newpipe[0], pid);
				return returnValue;
				break;
			case LOCAL:
				returnValue = SetLocalNEK(data, newpipe[0], newNek);
				CloseAll(newpipe[0], pid);
				return returnValue;
				break;		
			case REMOTE:
				returnValue = SetRemoteNEK(data, newpipe[0], newNek, remoteDek);
				CloseAll(newpipe[0], pid);
				return returnValue;
				break;
		}
	}
	return 0;
}

/*
 *	Entry point of the main process
 */
int main(int argc, char *argv[])
{

	char buff[256];
	int cont = TRUE;

	printf("\n\t======= dLANsoftware %s =======\n\n",DLANSOFTWARE_VERSION);
	// Check arguments number
	if (argc != 2)
	{
		Usage(argv[0]);
		return 0;
	}
	if(!strncmp(argv[1],"--h",3) || !strncmp(argv[1],"-h",2))
	{
		Usage(argv[0]);
		return 0;
	}
	// Copy the parameter to the local interface name
	strncpy(localifname, argv[1], MAX_INAME_LEN);
	// Check out root privileges
	if (getuid() != 0)
	{
		printf("\n\tTo run 'dlanconfig' you must have root privileges");
		fflush(stdout);
		return 0;
	}
	// Check the interface status and eventually bring it up
	if (InitIf(localifname, localMAC))
	{
		printf("\n\tError on the interface\n");
		return 1;
	}
	// Check local dLAN MAC: here it must begin the division depending on the device type...
	ForkProcesses(GETLOCALMAC, localifname, NULL, NULL);
	
	if (p_avTable.localBridge.isHere == 1)
	{
		printf("\n\tThe MAC of the local AV200 device is: ");
		PrintMAC(p_avTable.localBridge.MAC);
		printf("\n");
		while (cont)
		{
			PrintMenu();
			fgets((char*)buff, 256, stdin);
			switch ((char)buff[0])
			{
				case LOCAL:
				{
					char netPassword[PASSWDMAXLEN] = "";
					printf("\n\t%s\n", LOCALstr);
					printf("\tWrite the new local network password to set: ");
					fgets(netPassword, PASSWDMAXLEN, stdin);
					netPassword[strlen(netPassword) - 1] = '\0';
					if ((ForkProcesses(LOCAL, localifname, (unsigned char*)netPassword, NULL)) > 0)
						printf("\tSuccessfull!\n");
					break;
				}
				case REMOTE:
				{
					int tryfor = 4;
					char netPassword[PASSWDMAXLEN] = "";
					char devicePassword[PASSWDMAXLEN] = "";
					printf("\n\tSetting local and remote network passwords");
					printf("\n\tEnter the new network password to set on local and remote devices: ");
					fgets(netPassword, PASSWDMAXLEN, stdin);
					netPassword[strlen(netPassword) - 1] = '\0';
					
					while (tryfor > 0)
					{
						printf("\n\tInsert the security ID of the remote device: ");
						fgets(devicePassword, PASSWDMAXLEN, stdin);
						devicePassword[strlen(devicePassword) - 1] = '\0';
						if ((strlen(devicePassword) == 19) && (devicePassword[4] == '-') && (devicePassword[9] == '-') && (devicePassword[14] == '-'))
						{
							tryfor = -1;					
						}
						else
						{
							tryfor--;
							printf("\n\tError: the security ID has the form XXXX-XXXX-XXXX-XXXX\n");
						}
					}
					
					if (tryfor == -1)
					{
						printf("\n\tSetting local and remote network passwords");
						if ((ForkProcesses(REMOTE, localifname, (unsigned char*)netPassword, (unsigned char*)devicePassword)) > 0)
						{
							// Sets the local key to complete the operation
							if ((ForkProcesses(LOCAL, localifname, (unsigned char*)netPassword, NULL)) > 0)
								printf("\n\tSuccessfull!\n");
							
						}
						printf("\n\tWarning: you local password was set to %s\n", netPassword);
					}
					break;
				}
				case POLL:
				{
					ClearRemoteAV200Table();
					ForkProcesses(POLL, localifname, NULL, NULL);	
					PrintNodeAV200(localifname);
					break;
				}
				case EXIT:
				{
					if (localifstatus == DOWN)
						IfDown(localifname);
					cont = FALSE;
					break;
				}
			}
		}
	} else
	{
		printf("\tThe MAC of the local dLAN device is: ");
		PrintMAC(p_devTable.locBridge.MAC);
		printf("\n");
		while (cont)
		{
			PrintMenu();
			fgets((char*)buff, 256, stdin);
			switch ((char)buff[0])
			{
				case LOCAL:
				{
					char netPassword[PASSWDMAXLEN] = "";
					printf("\n\t%s\n", LOCALstr);
					printf("\tWrite the new local network password to set: ");
					fgets(netPassword, PASSWDMAXLEN, stdin);
					netPassword[strlen(netPassword) - 1] = '\0';
					if (!ForkProcesses(LOCAL, localifname, (unsigned char*)netPassword, NULL))
						printf("\tSuccessfull!\n");
					break;
				}
				case REMOTE:
				{
					int tryfor = 4;
					char netPassword[PASSWDMAXLEN] = "";
					char devicePassword[PASSWDMAXLEN] = "";
					printf("\n\t%s\n", REMOTEstr);
					printf("\n\tEnter the new network password to set on local and remote devices: ");
					fgets(netPassword, PASSWDMAXLEN, stdin);
					netPassword[strlen(netPassword) - 1] = '\0';
					while (tryfor > 0)
					{
						printf("\n\tInsert the security ID of the remote device: ");
						fgets(devicePassword, PASSWDMAXLEN, stdin);
						devicePassword[strlen(devicePassword) - 1] = '\0';
						if ((strlen(devicePassword) == 19) && (devicePassword[4] == '-') && (devicePassword[9] == '-') && (devicePassword[14] == '-'))
						{
							tryfor = -1;					
						}
						else
						{
							tryfor--;
							printf("\n\tError: the security ID has the form XXXX-XXXX-XXXX-XXXX\n");
						}
					}
					
					if (tryfor == -1)
					{
						printf("\tSetting local network password. ");
						if (!ForkProcesses(LOCAL, localifname, (unsigned char*)netPassword, NULL))
						{
							printf("\n\tSuccessfull!");
							printf("\n\tSetting remote network password");
							if (!ForkProcesses(REMOTE, localifname, (unsigned char*)netPassword, (unsigned char*)devicePassword))
							{
								printf("\n\tSuccessfull!\n\tRescanning the network\n");
								ClearRemoteDevoloTable();
								if (!ForkProcesses(POLL, localifname, NULL, NULL))
								{			
									printf("\n\tThe MAC of the local dLAN device is: ");
									PrintMAC(p_devTable.locBridge.MAC);
									printf("\n");
									PrintNode(localifname);
								}
								
							}
							printf("\n\tWarning: you local password was set to %s\n", netPassword);
						}
					}
					break;
				}
				case POLL:
				{
					ClearRemoteDevoloTable();
					ForkProcesses(POLL, localifname, NULL, NULL);	
					PrintNode(localifname);
					break;
				}
				case EXIT:
				{
					if (localifstatus == DOWN)
						IfDown(localifname);
					cont = FALSE;
					break;
				}
			}
		}
	}
	return 0;
}
