/* 
   dlanconfig.h: DLANConfig tool for HomePlug devices for linux

   Copyrighted 2003 by devolo AG

   Redistribution in source form, with or without modification,
   are NOT permitted.
   This program uses explicitly confidential information
   about HomePlug standard.
	    
*/
#ifndef DLANCONFIG_H
#define DLANCONFIG_H
/*
 *   Include config.h that ./configure generates
 *   and check if all function are present
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#ifndef HAVE_DUP2
#  error "No dup2 function."
#endif
#ifndef HAVE_STRERROR
#  error "No strerror function."
#endif
#ifndef HAVE_SELECT
#  error "No select function."
#endif
#ifndef HAVE_MEMSET
#  error "No memset function."
#endif
#ifndef HAVE_SOCKET
#  error "No socket function."
#endif
#ifndef HAVE_VFORK
#  ifndef HAVE_FORK
#     error "No fork function."
#  else
#     define vfork fork
#  endif
#endif
#endif


/*
 *   Max password length
 */
#define PASSWDMAXLEN 56

/*
 *  HomePlug and MME specific constants
 */
#define PACKET_HEADERSIZE         0x5
#define ETHERTYPE_INTELLON	  0x887B  // Ethernet Type 
#define	CODE_ReqStats	          0x07    // Request Stats Command       
#define	CODE_ReqNetStats          0x1a	  // Request Net Stats Command   
#define CODE_SetLocPar            0x19    // Set Local Parameters
#define	CODE_ReqStatsResp         0x08	  // Request Stats Response	
#define	CODE_ReturnNetStats       0x80	  // Return Network Stats Cmd    

/*VIPER*/
#define	CODE_PrivateVendorSpec    0x02	  // Private Vendor Specific
#define	CODE_HPSetNEKResp         0x06	  // set local nek response viper specific


#define SOCK_PROTO                0x1000  // Protocol number

#define	MME_ReqStatsSize          0                                       
#define MME_LocalParExtSize       0xF       
#define MME_MIN_FRAME_LEN         64      
#define MME_NET_STAT_FRAME_LEN    204     
#define MME_HDR_LEN               14      

/*
 *  MAC and DEK size
 */
#define MACSIZE                   6       
#define DECKSIZE                  8       

/*
 *   Max number of remote devices and 
 *   local interfaces
 */
#define MAX_BRIDGED_DEVS          40      // Maximum number of device
#define MAX_NET_INTERFACE_NUM     16      

/*
 *  Command retry constant
 */
#define POLL_RETRY                1       
#define SET_LOC_NEK_RETRY         1       
#define SET_REMOTE_NEK_RETRY      1       

/*
 *  Delay and timeout constant
 */
#define INTER_FRAME_DELAY         1       
#define EXTRA_WAIT_TIME_S         1       
#define WAIT_AFTER_NOACTIVITY     1       
#define SELECT_TIMEOUT_S          0       
#define SELECT_TIMEOUT_US         500000  

/*
 *  Specific length
 */
#define MAX_PACK_LEN              1500
#define HDR_LEN                   4
#define MAX_PACKET_TO_PARSE       30      
#define MAX_INAME_LEN             32      

#define DUMP_UNKNOWN_INTELLON_FRAME 1 
#define DUMP_INFO_LEVEL           0             

/*
 *  Constants used in rate calculation
 */
#define	M_RATE                    ( (588.0 - 38.0) / 481.0 )
#define	B_RATE                    ( 14.0 - M_RATE * (519.0/42.0) )

/*
 * This struct describes our local interface
 */
typedef struct NetDev {
  int index;      
  char iname[MAX_INAME_LEN];
  unsigned char mac[MACSIZE];
}NetDev_t;

/*
 * This struct describes a generic device
 */
typedef struct devolo_node
{
  unsigned char MAC[MACSIZE]; 
  unsigned char DEK[DECKSIZE]; 
  unsigned char NEK[DECKSIZE];
  double rate;                
  short toggled;              
  //  unsigned short devType1;
  unsigned char  devType;
  unsigned char  isHere;
} devolo_node_t;


/*
 * This struct contains all info we need
 * 
 */
typedef struct devolo_table{
  int devCounter;  
  struct devolo_node locBridge;
  struct devolo_node list[MAX_BRIDGED_DEVS];
}devolo_table_t;


/*
 *  APIs and specific structures
 */

int API_DoAction(int action,char* iname,char* newNek,char* remotePasswd);
struct devolo_table* API_Alloc_DevTable(void);
void API_PrintNode();

typedef struct API_NetW_DA{
  unsigned char	DA[MACSIZE]; 
  unsigned short  BYTES40;
  unsigned short 	FAILS;
  unsigned short 	DROPS;
} __attribute__ ((packed)) API_NetW_DA_t;

#define NWSIZE   15

typedef struct API_ReqNetStatsResp{
  unsigned char   DA[MACSIZE]; 
  unsigned char   SA[MACSIZE];	
  unsigned short  Ethertype;
  unsigned char	  MCTRL;	
  unsigned char	  MEHDR;
  unsigned char	  MELEN;
  unsigned char	  NETW_CTRL;
  unsigned short  BYTES40_ROBO;
  unsigned short  FAILS_ROBO;
  unsigned short  DROPS_ROBO;
  API_NetW_DA_t	  NW[NWSIZE];
}  __attribute__ ((packed))API_ReqNetStatsResp_t;


/*
 *   Specific MME structures
 */
typedef struct MME_PackedHdr{
  unsigned char   DA[MACSIZE];
  unsigned char   SA[MACSIZE];
  unsigned short  Ethertype;  
  unsigned char	  MCTRL ;     
  unsigned char	  MEHDR;      
  unsigned char	  MELEN;      
  unsigned char   NETW_CTRL;  
} __attribute__ ((packed))MME_PackedHdr_t;


typedef struct mme_hdr{
  unsigned char	  MCTRL ;
  unsigned char	  MEHDR; 
  unsigned char	  MELEN; 
  unsigned char   NETW_CTRL;
} __attribute__ ((packed))mme_hdr_t;


typedef struct mme_locParExtReq{
  unsigned char	  MCTRL ;         
  unsigned char	  MAC[MACSIZE];   
  unsigned char	  DECK[DECKSIZE]; 
  unsigned char   NETW_CTRL;      
} __attribute__ ((packed))mme_locParExtReq_t;


typedef struct mme_locParExtResp 
{
  unsigned char   DA[MACSIZE]; 
  unsigned char   SA[MACSIZE];	
  unsigned short  Ethertype;
  unsigned char	  MCTRL; 	
  unsigned char	  MEHDR;	
  unsigned char	  MELEN;
  unsigned char	  CTRL ;         
  unsigned char	  MAC[MACSIZE]; 
  unsigned char	  DEK[DECKSIZE];
  unsigned char   EKS;          
  unsigned char   NEK[DECKSIZE];
  unsigned char   TXCHAR[3];    
  unsigned char   OVERRIDES[22];
  unsigned char   RST_CNT[2];   
} __attribute__ ((packed))mme_locParExtResp_t;



typedef struct mme_single{ 
  unsigned short  Ethertype;
  unsigned char	  CTRL;
  unsigned char	  HDR;	 
  unsigned char	  LEN;  
  unsigned char   EKS;  
  unsigned char   NEK[DECKSIZE];
} __attribute__ ((packed))mme_single_t;


typedef struct mme_double{ 
  unsigned char	  CTRL;  
  unsigned char	  HDR1;	 
  unsigned char	  LEN1;  
  unsigned char   EKS;   
  unsigned char   NEK[DECKSIZE];
  unsigned char	  HDR2;	        
  unsigned char	  LEN2;         
  unsigned char   TXCHAR[3];    
} __attribute__ ((packed))mme_double_t;



typedef struct mme_triple{
  unsigned char	  CTRL;  
  unsigned char	  HDR1;	 
  unsigned char	  LEN1;  
  unsigned char   EKS1;  
  unsigned char   NEK1[DECKSIZE];
  unsigned char	  HDR2;	         
  unsigned char	  LEN2;          
  unsigned char   EKS2;          
  unsigned char   NEK2[DECKSIZE];
  unsigned char	  HDR3;	         
  unsigned char	  LEN3;          
  unsigned char   TXCHAR[3];     
} __attribute__ ((packed))mme_triple_t;


/*VIPERLINK*/

typedef unsigned char BYTE;

typedef struct stringdata
{
  BYTE Len;
  BYTE String[1];
}__attribute__ ((packed)) stringdata_t;

typedef struct Vendor
{
  BYTE flags;
  BYTE MACVersion[4];
  stringdata_t Mfg;
  stringdata_t Product;
}__attribute__ ((packed)) vendor_t;

//---------------------- for Viper ---------------------------------------


#define ISOLDDEVICE 1
#define ISVIPER 2
#define INTER_FRAME_DELAY_USEC   100000       
#define LOCAL_POLL_RETRY              2


#define DEVICE_DESCRIPTION_RESPONSE 0x10      
#define QUALITY_RESPONSE            0x18

typedef struct RXQUALITYINFO
{
  BYTE Address[MACSIZE];
  BYTE BytePer40[2];
  BYTE SignalPower[3];
  BYTE NoisePower[3];
}__attribute__ ((packed)) rxqualityinfo_t;

typedef struct TXQUALITYINFO
{
  BYTE Address[MACSIZE];
  unsigned short BytePer40;
}__attribute__ ((packed)) txqualityinfo_t;

//--------------------------------------------
// RTD created stucts
typedef struct private_mme
{
  //  unsigned char DA[MACSIZE]; 
  //  unsigned char SA[MACSIZE];    // local MAC Address
  //  unsigned short etherType;
  unsigned char numEntries;  // # MAC Management Entries 
  unsigned char MMEHeader;
  unsigned char MMELength;
  unsigned char MMEOUI[3];
  unsigned char MID;

  // various specific data types follow for each of the MIDs
  union
  {
    vendor_t vendor;  // Vendor MME
    rxqualityinfo_t rxData[1];    // array of qualities
    rxqualityinfo_t txData[1];
    BYTE data[1];  //general private form- some data bytes
  };
}__attribute__ ((packed))private_mme_t;

typedef struct private_mme_rsp
{
  unsigned char DA[MACSIZE]; 
  unsigned char SA[MACSIZE];    // local MAC Address
  unsigned short etherType;
  unsigned char numEntries;  // # MAC Management Entries 
  unsigned char MMEHeader;
  unsigned char MMELength;
  unsigned char MMEOUI[3];
  unsigned char MID;

  // various specific data types follow for each of the MIDs
  union
  {
    vendor_t vendor;  // Vendor MME
    rxqualityinfo_t rxData[1];    // array of qualities
    txqualityinfo_t txData[1];
    BYTE data[1];  //general private form- some data bytes
  };
}__attribute__ ((packed)) private_mme_rsp_t;



typedef struct MAC_MME_MSG
{
  //  unsigned char  mu8DA[MACSIZE];
  //  unsigned char  mu8SA[MACSIZE];
  //  unsigned short mu16Type;
  unsigned char  mu8Ctrl;
  unsigned char  mu8Header;
  unsigned char  mu8Len;
  unsigned char  mu8MsgID;
  unsigned char  mu8EKS;  //mu8Data[48]
  unsigned char  mu8DEK[8];
  unsigned char  muData[41];
} __attribute__ ((packed)) MAC_MME_MSG_t;


typedef struct mme_setremoteviper{ 
  unsigned short  Ethertype;
  unsigned char	  CTRL;
  unsigned char	  HDR;	 
  unsigned char	  LEN;  
  unsigned char   EKS;  
  unsigned char   NEK[DECKSIZE];
  unsigned char	  HDR2;	 
  unsigned char	  LEN2;  
  unsigned char   DUM2;  
} __attribute__ ((packed))mme_setremoteviper_t;


//---------------------------- For AV200 ----------------------------------------------------------
#define ISAV200 3
#define ETHERTYPE_AV200 0x88E1
#define MME_VERSION_AV200 0x00



enum { eCC = 0x0000, eVS = 0xA000 };
enum { eReq = 0x000, eCnf = 0x0001};

const unsigned char cIntellonOUI[3] = { 0x00, 0xb0, 0x52 };
const unsigned char default_destination_MAC[MACSIZE] = { 0x00, 0xb0, 0x52, 0x00, 0x00, 0x01 };

// VS_DISCOVER
enum { VS_DISCOVER_MMEReq = 0xA000, VS_DISCOVER_MMECnf = 0xA001 };
enum { VS_NETINFO_MMEReq = 0xA038, VS_NETINFO_MMECnf = 0xA039 };
enum { VS_SETKEY_MMEReq = 0xA050, VS_SETKEY_MMECnf = 0xA051 };

enum { eSUCCESS = 0x00, eFAIL = 0x01, eINVALID_PEKS = 0x10, eINVALID_PIB = 0x11, eINVALID_PEKS_PAYLOAD = 0x12, eREMOTE_FAIL = 0x13, eINVALID_REMOTE_ANSWER = 0x14, eREMOTE_DECRYPTION_FAILED = 0x15 };


/*
 *	This struct contains info about generic AV200 device
 */
typedef struct AV200_node
{
	unsigned char MAC[MACSIZE];
	unsigned char TEI;
	unsigned char FirstBridgedMAC[MACSIZE];
	unsigned char avgTxPhyRateMbps;
	unsigned char avgRxPhyRateMbps;
  	unsigned char isHere;

} AV200_node_t;


/*
 * 	This struct contains all info we need
 * 
 */
typedef struct AV200_table{
  	int devCounter;  
  	struct AV200_node localBridge;
  	struct AV200_node list[MAX_BRIDGED_DEVS];
} AV200_table_t;


struct AV200_MME
{
	unsigned char MMVersion;
	unsigned char MMEType[2];
	unsigned char IntellonOUI[3];
};


typedef struct AV200_MME_Header
{
	unsigned char DestinationAddress[MACSIZE];
	unsigned char SourceAddress[MACSIZE];
	unsigned char EthernetType[2];
	unsigned char MMVersion;
	unsigned char MMEType[2];
	unsigned char IntellonOUI[3];
} __attribute__ ((packed))AV200_MME_Header_t;


typedef struct AV200_MME_VendorSpecific
{
	struct AV200_MME_Header header;
	unsigned char data[53];

} __attribute__ ((packed))AV200_MME_VendorSpecific_t;


typedef struct AV200_MME_StationInfo
{
	unsigned char address[MACSIZE];
	unsigned char TEI;
	unsigned char firstBridgedAddress[MACSIZE];
	unsigned char avgTxPhyRateMbps;
	unsigned char avgRxPhyRateMbps;

} __attribute__ ((packed))AV200_MME_StationInfo_t;


typedef struct AV200_MME_NetworkInfo
{
	struct AV200_MME_Header header;
	unsigned char numberAVLans;	// Number of AV lans
	unsigned char networkID[7];	// Network ID
	unsigned char SNID;
	unsigned char TEI;
	unsigned char stationRole;
	unsigned char MAC[MACSIZE];
	unsigned char ccoTEI;
	unsigned char numStations;	// Number of station in the same lan

} __attribute__ ((packed))AV200_MME_NetworkInfo_t;


typedef struct AV200_MME_SetKey_Body
{
	unsigned char uPEKS;                // set to eKeyTypeNMK
	unsigned char cKEY[16];
	unsigned char uPEKS_EncPayload;     // set to eKeyTarget*
	unsigned char cRemoteMacAddr[6];    // set to broadcast addr
	unsigned char cKEY_EncPayload[16];  //

} __attribute__ ((packed))AV200_MME_SetKey_Body_t;

typedef struct AV200_MME_SetKey
{
	struct AV200_MME header;
	struct AV200_MME_SetKey_Body body;

} __attribute__ ((packed))AV200_MME_SetKey_t;


typedef struct AV200_MME_SetKeyCnf
{
	struct AV200_MME_Header header;
	unsigned char uStatus;      // Success = 0

} __attribute__ ((packed))AV200_MME_SetKeyCnf_t;

#endif


