
#include "sha256.h"


static const size_t SALT_SIZE = 8;
static const size_t MAX_SECRET_SIZE = 64;

const unsigned char DAKSalt[] = {0x08, 0x85, 0x6D, 0xAF, 0x7C, 0xF5, 0x81, 0x85};
const unsigned char NMKSalt[] = {0x08, 0x85, 0x6D, 0xAF, 0x7C, 0xF5, 0x81, 0x86};

#define GET_UINT32(n,b,i)                       \
{                                               \
    (n) = ( (unsigned long) (b)[(i)    ] << 24 )       \
        | ( (unsigned long) (b)[(i) + 1] << 16 )       \
        | ( (unsigned long) (b)[(i) + 2] <<  8 )       \
        | ( (unsigned long) (b)[(i) + 3]       );      \
}

#define PUT_UINT32(n,b,i)                       \
{                                               \
    (b)[(i)    ] = (unsigned char) ( (n) >> 24 );       \
    (b)[(i) + 1] = (unsigned char) ( (n) >> 16 );       \
    (b)[(i) + 2] = (unsigned char) ( (n) >>  8 );       \
    (b)[(i) + 3] = (unsigned char) ( (n)       );       \
}

unsigned char hash_value[32];

static unsigned char sha256_padding[64] =
{
 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
/*
 *	Initialize the context
 */
void InitializeSha256Context( sha256context_t *context)
{

	memset(context->total,0,sizeof(context->total));
	memset(context->state,0,sizeof(context->state));
	memset(context->buffer,0,sizeof(context->buffer));
	memset(context, 0, sizeof(sha256context_t));
        context->state[0] = 0x6A09E667;
	//printf(" %x ", context->state[0]);
        context->state[1] = 0xBB67AE85;
	//printf(" %x ", context->state[1]);
        context->state[2] = 0x3C6EF372;
	//printf(" %x ", context->state[2]);
        context->state[3] = 0xA54FF53A;
	//printf(" %x ", context->state[3]);
        context->state[4] = 0x510E527F;
	//printf(" %x ", context->state[4]);
        context->state[5] = 0x9B05688C;
	//printf(" %x ", context->state[5]);
        context->state[6] = 0x1F83D9AB;
	//printf(" %x ", context->state[6]);
        context->state[7] = 0x5BE0CD19;
	//printf(" %x ", context->state[7]);
}

void InitializeSaltedSecret(saltedSecret_t *secret, const unsigned char *iSecret, const unsigned char *iSalt)
{
	unsigned char l = ' ';

	secret->length = 0;
	memset( secret->value, 0, sizeof(secret->value));

	if (iSecret)
	{
		l = (unsigned char)strlen((char*)iSecret);
		if ( l > MAX_SECRET_SIZE)
		{
			l = MAX_SECRET_SIZE;
		}
		memcpy(secret->value, (unsigned char*)iSecret, l);
	}
	if (!iSalt)
	{
		l = 16;
	}
	secret->length = (unsigned char) (secret->length + l);
	if (iSalt)
	{
		memcpy(&secret->value[secret->length], (unsigned char*)iSalt, SALT_SIZE);
		secret->length += SALT_SIZE;
	}
	
}

void InitializeSaltedNMK( saltedSecret_t *secret, const unsigned char *iSecret)
{
	InitializeSaltedSecret(secret, iSecret, NMKSalt);
}

void InitializeSaltedDAK( saltedSecret_t *secret, const unsigned char *iSecret)
{
	InitializeSaltedSecret(secret, iSecret, DAKSalt);
}

void InitializeSaltedNID( saltedSecret_t *secret, const unsigned char *iSecret)
{
	InitializeSaltedSecret(secret, iSecret, 0);	
}

void ProcessContext( sha256context_t *context, unsigned char data[64])
{
	unsigned long temp1, temp2, W[64];
	unsigned long A, B, C, D, E, F, G, H;
	//
	GET_UINT32( W[0],  data,  0 );
	GET_UINT32( W[1],  data,  4 );
	GET_UINT32( W[2],  data,  8 );
	GET_UINT32( W[3],  data, 12 );
	GET_UINT32( W[4],  data, 16 );
	GET_UINT32( W[5],  data, 20 );
	GET_UINT32( W[6],  data, 24 );
	GET_UINT32( W[7],  data, 28 );
	GET_UINT32( W[8],  data, 32 );
	GET_UINT32( W[9],  data, 36 );
	GET_UINT32( W[10], data, 40 );
	GET_UINT32( W[11], data, 44 );
	GET_UINT32( W[12], data, 48 );
	GET_UINT32( W[13], data, 52 );
	GET_UINT32( W[14], data, 56 );
	GET_UINT32( W[15], data, 60 );
	
	#define  SHR(x,n) ((x & 0xFFFFFFFF) >> n)
	#define ROTR(x,n) (SHR(x,n) | (x << (32 - n)))
	
	#define S0(x) (ROTR(x, 7) ^ ROTR(x,18) ^  SHR(x, 3))
	#define S1(x) (ROTR(x,17) ^ ROTR(x,19) ^  SHR(x,10))
	
	#define S2(x) (ROTR(x, 2) ^ ROTR(x,13) ^ ROTR(x,22))
	#define S3(x) (ROTR(x, 6) ^ ROTR(x,11) ^ ROTR(x,25))
	
	#define F0(x,y,z) ((x & y) | (z & (x | y)))
	#define F1(x,y,z) (z ^ (x & (y ^ z)))
	
	#define R(t)                                    \
	(                                               \
	W[t] = S1(W[t -  2]) + W[t -  7] +          \
		S0(W[t - 15]) + W[t - 16]            \
	)
	
	#define P(a,b,c,d,e,f,g,h,x,K)                  \
	{                                               \
	temp1 = h + S3(e) + F1(e,f,g) + K + x;      \
	temp2 = S2(a) + F0(a,b,c);                  \
	d += temp1; h = temp1 + temp2;              \
	}
	
	A = context->state[0];
	B = context->state[1];
	C = context->state[2];
	D = context->state[3];
	E = context->state[4];
	F = context->state[5];
	G = context->state[6];
	H = context->state[7];
	
	P( A, B, C, D, E, F, G, H, W[ 0], 0x428A2F98 );
	P( H, A, B, C, D, E, F, G, W[ 1], 0x71374491 );
	P( G, H, A, B, C, D, E, F, W[ 2], 0xB5C0FBCF );
	P( F, G, H, A, B, C, D, E, W[ 3], 0xE9B5DBA5 );
	P( E, F, G, H, A, B, C, D, W[ 4], 0x3956C25B );
	P( D, E, F, G, H, A, B, C, W[ 5], 0x59F111F1 );
	P( C, D, E, F, G, H, A, B, W[ 6], 0x923F82A4 );
	P( B, C, D, E, F, G, H, A, W[ 7], 0xAB1C5ED5 );
	P( A, B, C, D, E, F, G, H, W[ 8], 0xD807AA98 );
	P( H, A, B, C, D, E, F, G, W[ 9], 0x12835B01 );
	P( G, H, A, B, C, D, E, F, W[10], 0x243185BE );
	P( F, G, H, A, B, C, D, E, W[11], 0x550C7DC3 );
	P( E, F, G, H, A, B, C, D, W[12], 0x72BE5D74 );
	P( D, E, F, G, H, A, B, C, W[13], 0x80DEB1FE );
	P( C, D, E, F, G, H, A, B, W[14], 0x9BDC06A7 );
	P( B, C, D, E, F, G, H, A, W[15], 0xC19BF174 );
	P( A, B, C, D, E, F, G, H, R(16), 0xE49B69C1 );
	P( H, A, B, C, D, E, F, G, R(17), 0xEFBE4786 );
	P( G, H, A, B, C, D, E, F, R(18), 0x0FC19DC6 );
	P( F, G, H, A, B, C, D, E, R(19), 0x240CA1CC );
	P( E, F, G, H, A, B, C, D, R(20), 0x2DE92C6F );
	P( D, E, F, G, H, A, B, C, R(21), 0x4A7484AA );
	P( C, D, E, F, G, H, A, B, R(22), 0x5CB0A9DC );
	P( B, C, D, E, F, G, H, A, R(23), 0x76F988DA );
	P( A, B, C, D, E, F, G, H, R(24), 0x983E5152 );
	P( H, A, B, C, D, E, F, G, R(25), 0xA831C66D );
	P( G, H, A, B, C, D, E, F, R(26), 0xB00327C8 );
	P( F, G, H, A, B, C, D, E, R(27), 0xBF597FC7 );
	P( E, F, G, H, A, B, C, D, R(28), 0xC6E00BF3 );
	P( D, E, F, G, H, A, B, C, R(29), 0xD5A79147 );
	P( C, D, E, F, G, H, A, B, R(30), 0x06CA6351 );
	P( B, C, D, E, F, G, H, A, R(31), 0x14292967 );
	P( A, B, C, D, E, F, G, H, R(32), 0x27B70A85 );
	P( H, A, B, C, D, E, F, G, R(33), 0x2E1B2138 );
	P( G, H, A, B, C, D, E, F, R(34), 0x4D2C6DFC );
	P( F, G, H, A, B, C, D, E, R(35), 0x53380D13 );
	P( E, F, G, H, A, B, C, D, R(36), 0x650A7354 );
	P( D, E, F, G, H, A, B, C, R(37), 0x766A0ABB );
	P( C, D, E, F, G, H, A, B, R(38), 0x81C2C92E );
	P( B, C, D, E, F, G, H, A, R(39), 0x92722C85 );
	P( A, B, C, D, E, F, G, H, R(40), 0xA2BFE8A1 );
	P( H, A, B, C, D, E, F, G, R(41), 0xA81A664B );
	P( G, H, A, B, C, D, E, F, R(42), 0xC24B8B70 );
	P( F, G, H, A, B, C, D, E, R(43), 0xC76C51A3 );
	P( E, F, G, H, A, B, C, D, R(44), 0xD192E819 );
	P( D, E, F, G, H, A, B, C, R(45), 0xD6990624 );
	P( C, D, E, F, G, H, A, B, R(46), 0xF40E3585 );
	P( B, C, D, E, F, G, H, A, R(47), 0x106AA070 );
	P( A, B, C, D, E, F, G, H, R(48), 0x19A4C116 );
	P( H, A, B, C, D, E, F, G, R(49), 0x1E376C08 );
	P( G, H, A, B, C, D, E, F, R(50), 0x2748774C );
	P( F, G, H, A, B, C, D, E, R(51), 0x34B0BCB5 );
	P( E, F, G, H, A, B, C, D, R(52), 0x391C0CB3 );
	P( D, E, F, G, H, A, B, C, R(53), 0x4ED8AA4A );
	P( C, D, E, F, G, H, A, B, R(54), 0x5B9CCA4F );
	P( B, C, D, E, F, G, H, A, R(55), 0x682E6FF3 );
	P( A, B, C, D, E, F, G, H, R(56), 0x748F82EE );
	P( H, A, B, C, D, E, F, G, R(57), 0x78A5636F );
	P( G, H, A, B, C, D, E, F, R(58), 0x84C87814 );
	P( F, G, H, A, B, C, D, E, R(59), 0x8CC70208 );
	P( E, F, G, H, A, B, C, D, R(60), 0x90BEFFFA );
	P( D, E, F, G, H, A, B, C, R(61), 0xA4506CEB );
	P( C, D, E, F, G, H, A, B, R(62), 0xBEF9A3F7 );
	P( B, C, D, E, F, G, H, A, R(63), 0xC67178F2 );
	
	context->state[0] += A;
	context->state[1] += B;
	context->state[2] += C;
	context->state[3] += D;
	context->state[4] += E;
	context->state[5] += F;
	context->state[6] += G;
	context->state[7] += H;

}

void UpdateContext( sha256context_t *context, unsigned char *input, unsigned long length)
{
	unsigned long left, fill;
	//
	if(!length)
	return;
	left = context->total[0] & 0x3F;
	fill = 64 - left;
	context->total[0] += length;
	context->total[0] &= 0xFFFFFFFF;
	if(context->total[0] < length)
		context->total[1]++;
	if(left && length >= fill)
	{
		memcpy((void*)(context->buffer + left), (void*)input, (unsigned short) fill);
		ProcessContext(context, context->buffer);
		length -= fill;
		input  += fill;
		left = 0;
	}
	while(length >= 64)
	{
	ProcessContext(context, input);
	length -= 64;
	input  += 64;
	}
	if(length) {
	memcpy((void*)(context->buffer + left), (void*)input, (unsigned short) length);
	}
}

void FinishContext(sha256context_t *context)
{
	// Locals
	unsigned long last, padn;
	unsigned long high, low;
	unsigned char msglen[8];
	//
	high = ( context->total[0] >> 29 ) | ( context->total[1] <<  3 );
	low  = ( context->total[0] <<  3 );
	
	PUT_UINT32( high, msglen, 0 );
	PUT_UINT32( low,  msglen, 4 );
	
	last = context->total[0] & 0x3F;
	padn = ( last < 56 ) ? ( 56 - last ) : ( 120 - last );
	
	UpdateContext(context, sha256_padding, padn);
	UpdateContext(context, msglen, 8);
	
	PUT_UINT32( context->state[0], hash_value,  0 );
	PUT_UINT32( context->state[1], hash_value,  4 );
	PUT_UINT32( context->state[2], hash_value,  8 );
	PUT_UINT32( context->state[3], hash_value, 12 );
	PUT_UINT32( context->state[4], hash_value, 16 );
	PUT_UINT32( context->state[5], hash_value, 20 );
	PUT_UINT32( context->state[6], hash_value, 24 );
	PUT_UINT32( context->state[7], hash_value, 28 );

}

const unsigned char* HashNMK(const unsigned char* iSecret)
{
	sha256context_t context;
	saltedSecret_t secret;
	int j;

	InitializeSha256Context(&context);
	memset(hash_value, 0x00, sizeof(hash_value));

	InitializeSaltedNMK(&secret, iSecret);
	UpdateContext(&context, secret.value, secret.length);
	FinishContext(&context);
	for(j = 0; j < 999; j++) {
		InitializeSha256Context(&context);
		UpdateContext(&context, hash_value, 32);
		FinishContext(&context);
	}
	
  	return hash_value;	
}

const unsigned char* HashDAK(const unsigned char* iSecret)
{
	sha256context_t context;
	saltedSecret_t secret;
	int j;

	InitializeSha256Context(&context);
	memset(hash_value, 0x00, sizeof(hash_value));

	InitializeSaltedDAK(&secret, iSecret);
	UpdateContext(&context, secret.value, secret.length);
	FinishContext(&context);
	for(j = 0; j < 999; j++) {
		InitializeSha256Context(&context);
		UpdateContext(&context, hash_value, 32);
		FinishContext(&context);
	}

  	return hash_value;
}

const unsigned char* HashNID(const unsigned char* iSecret)
{
	sha256context_t context;
	saltedSecret_t secret;
	int j;

	InitializeSha256Context(&context);
	memset(hash_value, 0x00, sizeof(hash_value));

	InitializeSaltedNID(&secret, iSecret);
	UpdateContext(&context, secret.value, secret.length);
	FinishContext(&context);
	for(j = 0; j < 4; j++) {
		InitializeSha256Context(&context);
		UpdateContext(&context, hash_value, 32);
		FinishContext(&context);
	}
	
  	return hash_value;
}

void NetworkKeyFromPassword(const unsigned char *password, unsigned char *key)
{
	unsigned char passwordCopy[MAX_SECRET_SIZE + 1];
	memcpy((unsigned char *)passwordCopy, password, MAX_SECRET_SIZE);
	
	const unsigned char *passwordHash;
	passwordHash = (const unsigned char *)malloc(32);
	passwordHash = HashNMK(passwordCopy);
	memcpy(key, passwordHash, 16); 
}

void DeviceAccessKeyFromSecurityID(const unsigned char *securityID, unsigned char *dak)
{
	unsigned char securityIDCopy[MAX_SECRET_SIZE + 1];
	memcpy((unsigned char *)securityIDCopy, securityID, MAX_SECRET_SIZE);
	
	const unsigned char *securityIDHash;
	securityIDHash = (const unsigned char *)malloc(32);
	securityIDHash = HashDAK(securityIDCopy);
	memcpy(dak, securityIDHash, 16); 
}

void dosomething(){ return;};




